/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: A_Changes_History.java,v 1.16 2001/10/05 14:43:41 cryd0221 Exp $
 */
package org.enhydra.barracuda;

/**
 * <p>The purpose of this class is to track all major changes made to 
 * the Barracuda framework. If you check changes into cvs, you must
 * document those changes here. The list should be sorted in reverse
 * chronological order (newer at the top).
 * 
 * <ul>
 * <li>
 *   <p><strong>csc_100501.1</strong> - Modified ApplicationGateway so
 *      that it creates the DispatchQueue using a factory method. This 
 *      makes it possible for someone to provide a different implementation.
 *      Thanks to Diez B. Roggisch [d.roggisch@artnology.com] for the suggestion.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_100401.1</strong> - Very minor change to remove some 
 *      vestige code from AbstractTemplateModel: basically it still had
 *      a resetModel() method in it, which went away from the interface
 *      long ago. I just overlooked it in AbstractTemplateModel.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_100301.2</strong> - Several changes.<p>
 *      First of all, there was a problem in the jsrsClientServerHTTPLib 
 *      script. Basically, it was not setting the selected options correctly.
 *      In addition, I modified the posted script to not include button, submit
 *      and reset elements. Furthermore, I made it so that when you run it
 *      with debuggin turned on, you can manually submit the generated script
 *      (in IE anyway). This fixes a problem whereby the values of select 
 *      elements were not getting passed through to the ParamGateway correctly.
 *   </p><p>
 *      Second, there was a problem in HTMLActionRenderer whereby if the 
 *      action included url params, when the back button was disabled it 
 *      would screw up the resulting URL that was created to map the request
 *      to the param gateway. There were 2 underlying problems. First of all,
 *      the BAction component when constructing an action String needed to be 
 *      encoding the parameter key/values. Second, when building the disabled
 *      action String, the params needed to go outside of the extension. To 
 *      facilitate this I created a LightweightURL class.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_100301.1</strong> - Backed out yesterday's changes
 *      for disabling a select element when rendering in HTMLActionRenderer.
 *      We can't do this for selects, since it causes the value not to be 
 *      submitted along with the form if it is disabled. Input and Button
 *      elements are still disabled.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_100201.2</strong> - Created a NamingHelper class that
 *      can be used to retrieve teh name of an HTMLElement. If the element
 *      supports the name attribute but the value is not set, a unique
 *      name will be automatically created. 
 *   </p>
 * </li>
 * <li>
 *   <p><strong>merg_100201.1</strong> - Added a close() statement after 
 *      reading the resource from the inputstream. I noticed that I 
 *      couldn't redeploy the app when it used the ResourceGateway. This 
 *      simple .close() fixes it.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_100201.1</strong> - When rendering BAction componnents,
 *      make sure we disable the element to which they are bound (for input,
 *      button, and select elements), thus preventing duplicate submits by 
 *      double clicking (note that this won't work in NN4)
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_100101.1</strong> - Modified the 
 *      DefaultApplicationAssembler so that it can read assembler files
 *      from either the /WEB-INF directory OR the /WEB-INF/classes dir;
 *      previously, files had to be in /WEB-INF/classes.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>merg_092901.1</strong> - HttpServletRequestWrapper:
 *      Some servlet containers do not accurately report all parameters 
 *      submitted with the request. If the request is a post, and there 
 *      were additional parameters submitted in the url, the additional 
 *      params are sometimes not returned (ie. on ATG Dynamo). SO...if 
 *      the value comes back null, then actually check the query string 
 *      for them.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>jk_092801.1</strong> - 
 *      Supplied a patch for DigitValidator that fixed a logic bug and did 
 *      general cleanup on the code.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_092801.1</strong> - 
 *      Moved FormGateway from comp.event to comp.helper
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_092701.1</strong> - 
 *      We had a problem in BText and BLink, in that the text was always
 *      getting placed into the DOM using a CDATASection. This meant that
 *      the text was not encoded, so if you returned a String with a markup
 *      symbol in it, like '<', it would hose the validity of your DOM.
 *   </p><p>
 *      Basically, the text component should generally encode text before 
 *      rendering. There are, however, exceptions to this rule, like when you 
 *      actually want to embed markup in some text.
 *   </p><p>
 *      The solution to this was to modify the DOMUtil.setTextInNode() method
 *      to take an extra parameter that indicates whether or not to encode. 
 *      Then, we modified BText and BLink to each have a setAllowMarkupInText()
 *      method...if set, then the text will not be encoded. The default, 
 *      however, is to always encode.
 *   </p><p>
 *      Finally, built a sample (CompEx7) to test it out, and a test case 
 *      (TestBTextRender) to validate it.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_092501.1</strong> - 
 *      Minor code cleanup--formatting changes, adding EPL headers, etc.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_092101.1</strong> - 
 *      fixed a problem in TemplateHelper where the BTemplate delegates that get created
 *      not only need to inherit the model information from their parent, but also the 
 *      view settings as well (ie. if the directives are coming from a properties file).
 *      The proper way to do this is to make views cloneable (thereby inheriting all their
 *      settings)
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_091801</strong> - 
 *      fixed EventBuilder so that it would compile under JDK 1.2.2; evidently there
 *      is a bug in this version of the JDK which prevents inner classes from accessing
 *      protected values in their parent class. Compiled fine in JDK 1.3.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_091701</strong> - 
 *      Created a new BTemplateGateway and BTemplateViewHandler to make it easier to 
 *      work with template views. BTemplateViewHandler extends DefaultViewHandler, 
 *      which in turn provides basic component lifecycle functionality (similar to that 
 *      of ComponentGateway) but for event driven processing. Basically, the idea is 
 *      that when you need to handle view events (and thus generate a view) this 
 *      helper class should handle 90% of the use cases. Thanks to 
 *      Timo Sillanp [Timo.Sillanpaa@reach-u.com] for suggesting this idea.
 *   </p><p>
 *      Reworked the SimpleLogin (examples/ex1) to use this approach (eliminated a 
 *      ton of code by doing so). Look here for examples.
 *   </p><p>
 *      Fixed a nasty Javascript bug in HTMLActionRenderer that was causing forms 
 *      to be submitted twice. Add convenience methods to SessionServices to make 
 *      it easy to get session info from a ViewContext reference. 
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_091401.1</strong> - 
 *      the purpose of this mod is to make it possible to process directives in 
 *      the root node of the template view. We can only do this for BTemplates 
 *      that are step children because in this case they will only be rendered
 *      once and thus its not a big deal to go replacing the root node
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_091301.2</strong> - 
 *      Updated TemplateHelper.java to support the notion of recursive template
 *      parsing. Basically, there are two known use cases where data coming back
 *      from a template model might actually contain additional directives which
 *      need to be parsed. In the first case, an anchor element might have a nested
 *      image; in the second case, a template model might return a chunk of markup
 *      (ie. to insert a header, footer, etc) which in turn might contain directives
 *      that need to be parsed.
 *   </p><p>
 *      In both of these cases now, child BTemplateComponents are created and 
 *      added to the parent component to ensure that the sub-nodes get processed as
 *      well. The second part of /Barracuda/CompTest6 verifies this. Also created
 *      a test case (TestBTemplateRender) to validate this.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_091301.1</strong> - 
 *      Updated TemplateHelper.java so that when a TemplateModel's getItem()
 *      method returns null it will skip that node entirely. Previously, a
 *      default "key not found" type message would be displayed, which is really
 *      actually redundant with the default implementation of the 
 *      AbstractTemplateModel. Created CompEx6.java to visually verify this.
 *   </p>
 * </li>
 * <li>
 *   <p><strong>csc_091301.0</strong> - 
 *      Created this document.
 *   </p>
 * </li>
 * </ul>
 */
public class A_Changes_History {

    private A_Changes_History() {
        super();
    }
    
    public static void main(String args[]) {
        System.out.println ("This class records the changes history to the Barracuda framework");
        System.out.println ("obj 1:"+new Object());
        System.out.println ("obj 2:"+new Object());        
    }
}