/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BarracudaConfig.java,v 1.13 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import java.lang.ref.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.dom.io.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.srv.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;

import org.enhydra.barracuda.config.events.*;
import org.enhydra.barracuda.config.xmlc.*;

/**
 * This mini-app is can be used to configure the core Barracuda 
 * classes. This class contains event handlers (both Controller 
 * and View) for the Config screen (which is the only screen)
 */
public class BarracudaConfig extends DefaultEventGateway {

    //public constants
    protected static BCategory logger = BCategory.getBInstance(BarracudaConfig.class.getName());

    public static final String MASTER_SCREEN = BarracudaConfig.class.getName()+".MasterScreen";
    public static final String FORM = BarracudaConfig.class.getName()+".Form";
    public static final String ERR = BarracudaConfig.class.getName()+".Errors";

    //this defines the various event handlers
    private ListenerFactory getConfigFactory =             new DefaultListenerFactory() {public BaseEventListener getInstance() {return new GetConfigHandler();}            public String getListenerID() {return getID(GetConfigHandler.class);}};
    private ListenerFactory getNextConfigFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new GetNextConfigHandler();}            public String getListenerID() {return getID(GetConfigHandler.class);}};
    private ListenerFactory renderConfigFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderConfigHandler();}        public String getListenerID() {return getID(RenderConfigHandler.class);}};                

    //------------------- BarracudaConfig ------------------------
    /**
     * Public constructor
     */
    public BarracudaConfig() {
        //specify who's interested in what
        specifyLocalEventInterests(getConfigFactory, GetBConfig.class);
        specifyLocalEventInterests(getNextConfigFactory, GetNextBConfig.class);
        specifyLocalEventInterests(renderConfigFactory, RenderBConfig.class);

        //add in sub-gateways
        this.add(new CompConfig());    
        this.add(new DataConfig());    
        this.add(new DomConfig());    
        this.add(new EventConfig());    
        this.add(new FormsConfig());    
        this.add(new UtilConfig());    
        this.add(new ViewConfig());    
        this.add(new MasterConfig());    
    }


    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * GetConfigHandler - handle the request to get the config 
     * screen.
     */
    class GetConfigHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //get the appropriate screen for the desired context and locale
            Locale locale = context.getViewCapabilities().getClientLocale();
            MasterScreen screen = new MasterScreenFactory().getInstance(BarracudaConfig.this, context, locale);
            
            //make sure the proper tab is selected (get curtab from session)
            HttpSession session = SessionServices.getSession(context.getRequest());
            String curtab = (String) session.getAttribute(TabsModel.CUR_TAB);
            screen.tabsModel.setCurrentTab(curtab);
            
            //place the screen in the context (so the renderer can access it)
            context.putState(MASTER_SCREEN, screen);

            //forward control to the renderer
            context.getQueue().addEvent(new RenderBConfig());
        }
    }

    /**
     * GetNextConfigHandler - get the next barracuda config screen (expects 
     * a TabsModel.CUR_TAB parameter in the request
     */
    class GetNextConfigHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //get the curtab parameter
            String curtab = context.getRequest().getParameter(TabsModel.CUR_TAB);
            
            //place it in the session (so it'll be available when we handle
            //the get event request)
            HttpSession session = SessionServices.getSession(context.getRequest());
            session.setAttribute(TabsModel.CUR_TAB, curtab);

            //now redirect to the main get event
            throw new ClientSideRedirectException(new GetBConfig());
        }
    }

    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderConfigHandler - handle the request to render the config 
     * screen
     */
    class RenderConfigHandler extends DefaultBaseEventListener {
        public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
            //get the screen from context
            MasterScreen screen = (MasterScreen) context.getState(MASTER_SCREEN);

            //grab the component hierarchy and render it
            try {screen.bcRoot.render(new DefaultViewContext(context));} 
            catch (RenderException e) {logger.debug ("Error rendering views:", e);}

            //now render the DOM
            //06_19_2001_Jacob_Kjome_start - hoju@visi.com
            //get the output options
            OutputOptions oo = DefaultDOMWriter.getDefaultOutputOptions(screen.dom.getOwnerDocument());

            //add doctype - need to do here because XMLC can't yet add the one embedded in the html to the DOM
            oo.setOmitDocType(false);
            oo.setPublicId("-//W3C//DTD HTML 4.01 Transitional//EN");
            oo.setSystemId("http://www.w3.org/TR/html401/loose.dtd");
                        
            //now render the DOM
//            new DefaultDOMWriter().write(screen.dom, context.getResponse());
            new DefaultDOMWriter().write(screen.dom, context.getResponse(), oo);
            //06_19_2001_Jacob_Kjome_end
        }
    }
}