/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: CompConfig.java,v 1.14 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.http.ServletUtil;
import org.enhydra.barracuda.core.util.logging.*;

import org.enhydra.barracuda.config.events.*;

/**
 * This class provides the methods needed to configure the component
 * screens
 */
public class CompConfig extends DefaultEventGateway {

    protected static BCategory logger = BCategory.getBInstance(CompConfig.class.getName());

    //form id (must be unique)
    public static final String FORM = CompConfig.class.getName()+".Form";

    //model name
    public static final String MODEL_NAME = "Comp";

    //model elements (these are the data items supported)
    //...components
    public static final String ABSTRACT_BCOMPONENT_DL = "AbstractBComponent_DebugLevel";
    public static final String BACTION_DL = "BAction_DebugLevel";
    public static final String BINPUT_DL = "BInput_DebugLevel";
    public static final String BLINK_DL = "BLink_DebugLevel";
    public static final String BLIST_DL = "BList_DebugLevel";
    public static final String BSELECT_DL = "BSelect_DebugLevel";
    public static final String BTABLE_DL = "BTable_DebugLevel";
    public static final String BTEMPLATE_DL = "BTemplate_DebugLevel";
    public static final String BTEXT_DL = "BText_DebugLevel";
    public static final String BTOGGLE_BUTTON_DL = "BToggleButton_DebugLevel";
    public static final String COMPONENT_GATEWAY_DL = "ComponentGateway_DebugLevel";
    //...models
    public static final String DEFAULT_LIST_MODEL_DL = "DefaultListModel_DebugLevel";
    public static final String DEFAULT_PROPERTIES_MODEL_DL = "DefaultPropertiesModel_DebugLevel";
    //...views
//    public static final String DEFAULT_LIST_VIEW_DL = "DefaultListView_DebugLevel";
    public static final String DEFAULT_TABLE_VIEW_DL = "DefaultTableView_DebugLevel";
    public static final String DEFAULT_TEMPLATE_VIEW_DL = "DefaultTemplateView_DebugLevel";
    public static final String DEFAULT_VIEW_DL = "DefaultView_DebugLevel";
    //...renderers
    public static final String HTML_COMPONENT_RENDERER_DL = "HTMLComponentRenderer_DebugLevel";
    public static final String HTML_ACTION_RENDERER_DL = "HTMLActionRenderer_DebugLevel";
    public static final String HTML_INPUT_RENDERER_DL = "HTMLInputRenderer_DebugLevel";
    public static final String HTML_LINK_RENDERER_DL = "HTMLLinkRenderer_DebugLevel";
    public static final String HTML_LIST_RENDERER_DL = "HTMLListRenderer_DebugLevel";
    public static final String HTML_SELECT_RENDERER_DL = "HTMLSelectRenderer_DebugLevel";
    public static final String HTML_TABLE_RENDERER_DL = "HTMLTableRenderer_DebugLevel";
    public static final String HTML_TOGGLE_RENDERER_DL = "HTMLToggleRenderer_DebugLevel";
    public static final String HTML_TEMPLATE_RENDERER_DL = "HTMLTemplateRenderer_DebugLevel";
    public static final String HTML_TEXT_RENDERER_DL = "HTMLTextRenderer_DebugLevel";
    public static final String TEMPLATE_HELPER_DL = "TemplateHelper_DebugLevel";
    public static final String XML_COMPONENT_RENDERER_DL = "XMLComponentRenderer_DebugLevel";
    public static final String XML_TEMPLATE_RENDERER_DL = "XMLTemplateRenderer_DebugLevel";
    public static final String XML_TEXT_RENDERER_DL = "XMLTextRenderer_DebugLevel";
    //...buttons
    public static final String ERROR_MESSAGE = "ErrorMessage";
    public static final String SUCCESS_MESSAGE = "SuccessMessage";
    public static final String UPDATE_BUTTON = "UpdateButton";

    //private vars
    private ListenerFactory updateConfigFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new UpdateConfigHandler();}        public String getListenerID() {return getID(UpdateConfigHandler.class);}};

    public CompConfig() {
        //specify generic interest
        specifyLocalEventInterests(updateConfigFactory);
    }

    //------------------------------------------------------------
    //                       Data Models
    //------------------------------------------------------------
    /**
     * define the template model that backs this screen
     */
    class CompModel extends AbstractTemplateModel {

        //register the model by name
        public String getName() {return MODEL_NAME;}
        
        //provide items by key
//        public Object getItem(String key, ViewContext vc) {
        public Object getItem(String key) {
            ViewContext vc = getViewContext();

            if (logger.isDebugEnabled()) logger.debug("Asking for key:"+key);    
            
            //Handle requests for data
            //...components
            if (key.equals(ABSTRACT_BCOMPONENT_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, AbstractBComponent.class);
            } else if (key.equals(BACTION_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BAction.class);
            } else if (key.equals(BINPUT_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BInput.class);
            } else if (key.equals(BLINK_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BLink.class);
            } else if (key.equals(BLIST_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BList.class);
            } else if (key.equals(BSELECT_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BSelect.class);
            } else if (key.equals(BTABLE_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BTable.class);
            } else if (key.equals(BTEMPLATE_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BTemplate.class);
            } else if (key.equals(BTEXT_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BText.class);
            } else if (key.equals(BTOGGLE_BUTTON_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, BToggleButton.class);
            } else if (key.equals(COMPONENT_GATEWAY_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, ComponentGateway.class);
            //...models
            } else if (key.equals(DEFAULT_LIST_MODEL_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultListModel.class);
            } else if (key.equals(DEFAULT_PROPERTIES_MODEL_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultPropertiesModel.class);
            //...views
            } else if (key.equals(DEFAULT_TABLE_VIEW_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultTableView.class);
            } else if (key.equals(DEFAULT_TEMPLATE_VIEW_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultTemplateView.class);
            } else if (key.equals(DEFAULT_VIEW_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, DefaultView.class);
            //...renderers
            } else if (key.equals(HTML_COMPONENT_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLComponentRenderer.class);
            } else if (key.equals(HTML_COMPONENT_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLComponentRenderer.class);
            } else if (key.equals(HTML_ACTION_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLActionRenderer.class);
            } else if (key.equals(HTML_INPUT_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLInputRenderer.class);
            } else if (key.equals(HTML_LINK_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLLinkRenderer.class);
            } else if (key.equals(HTML_LIST_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLListRenderer.class);
            } else if (key.equals(HTML_SELECT_RENDERER_DL)) { 
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLSelectRenderer.class);
            } else if (key.equals(HTML_TABLE_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLTableRenderer.class);
            } else if (key.equals(HTML_TOGGLE_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLToggleRenderer.class);
            } else if (key.equals(HTML_TEMPLATE_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLTemplateRenderer.class);
            } else if (key.equals(HTML_TEXT_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, HTMLTextRenderer.class);
            } else if (key.equals(TEMPLATE_HELPER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, TemplateHelper.class);
            } else if (key.equals(XML_COMPONENT_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, XMLComponentRenderer.class);
            } else if (key.equals(XML_TEMPLATE_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, XMLTemplateRenderer.class);
            } else if (key.equals(XML_TEXT_RENDERER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, XMLTextRenderer.class);
            //...buttons
            } else if (key.equals(ERROR_MESSAGE)) {
                return ScreenUtil.getErrMsg(vc, FORM, ERROR_MESSAGE);
            } else if (key.equals(SUCCESS_MESSAGE)) {
                return ScreenUtil.getSuccessMsg(vc, FORM, SUCCESS_MESSAGE);
            } else if (key.equals(UPDATE_BUTTON)) {
                return ScreenUtil.getUpdateButtonComp(vc, updateConfigFactory);
//            } else return super.getItem(key, vc);
            } else return super.getItem(key);
            
        }
    }

    //------------------------------------------------------------
    //                  Form Mappings, Validators
    //------------------------------------------------------------
    /**
     * define the form map that backs the model
     */
    class CompForm extends DefaultFormMap {
        public CompForm() {
            //define the elements (note: these don't need any validators). Note
            //also that we set the defaults to the current values. This allows
            //us just to reset all current values down below without checking to
            //see if the elements actually got passed in from the form.
            if (logger.isDebugEnabled()) logger.debug("Defining form elements");            
            //...components
            this.defineElement(new DefaultFormElement(ABSTRACT_BCOMPONENT_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(AbstractBComponent.class)), null, false));
            this.defineElement(new DefaultFormElement(BACTION_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BAction.class)), null, false));
            this.defineElement(new DefaultFormElement(BINPUT_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BInput.class)), null, false));
            this.defineElement(new DefaultFormElement(BLINK_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BLink.class)), null, false));
            this.defineElement(new DefaultFormElement(BLIST_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BList.class)), null, false));
            this.defineElement(new DefaultFormElement(BSELECT_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BSelect.class)), null, false));
            this.defineElement(new DefaultFormElement(BTABLE_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BTable.class)), null, false));
            this.defineElement(new DefaultFormElement(BTEMPLATE_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BTemplate.class)), null, false));
            this.defineElement(new DefaultFormElement(BTEXT_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BText.class)), null, false));
            this.defineElement(new DefaultFormElement(BTOGGLE_BUTTON_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(BToggleButton.class)), null, false));
            this.defineElement(new DefaultFormElement(COMPONENT_GATEWAY_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(ComponentGateway.class)), null, false));
            //...models
            this.defineElement(new DefaultFormElement(DEFAULT_LIST_MODEL_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(DefaultListModel.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_PROPERTIES_MODEL_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(DefaultPropertiesModel.class)), null, false));
            //...views
            this.defineElement(new DefaultFormElement(DEFAULT_TABLE_VIEW_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(DefaultTableView.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_TEMPLATE_VIEW_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(DefaultTemplateView.class)), null, false));
            this.defineElement(new DefaultFormElement(DEFAULT_VIEW_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(DefaultView.class)), null, false));
            //...renderers
            this.defineElement(new DefaultFormElement(HTML_COMPONENT_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLComponentRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_COMPONENT_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLComponentRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_ACTION_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLActionRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_INPUT_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLInputRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_LINK_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLLinkRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_LIST_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLListRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_SELECT_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLSelectRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_TABLE_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLTableRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_TOGGLE_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLToggleRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_TEMPLATE_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLTemplateRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(HTML_TEXT_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(HTMLTextRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(TEMPLATE_HELPER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(TemplateHelper.class)), null, false));
            this.defineElement(new DefaultFormElement(XML_COMPONENT_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(XMLComponentRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(XML_TEMPLATE_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(XMLTemplateRenderer.class)), null, false));
            this.defineElement(new DefaultFormElement(XML_TEXT_RENDERER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(XMLTextRenderer.class)), null, false));
        }
    }

    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * UpdateConfigHandler - handle the request to update the config 
     * screen.
     */
    class UpdateConfigHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //figure out our target locale and get the appropriate screen
            Locale locale = context.getViewCapabilities().getClientLocale();
            MasterScreen screen = new MasterScreenFactory().getInstance(CompConfig.this, context, locale);
            if (logger.isDebugEnabled()) ServletUtil.showParams(context.getRequest(), logger);
            
            //create the login form
            ValidationException ve = null;
            CompForm formMap = new CompForm();
            try {
                //map/validate the form
                formMap.map(context.getRequest()).validate(true);
                
                //If there were no errors, update the data. Note that we just
                //assume that all the data is here. The reason we can do this is
                //because we prepopulated the form up above with the default values.
                //The more proper way would be to check each value, see if it's set,
                //and then only update it if the value has actually changed. Lot more
                //code to do that though, and since we're just setting statics, the
                //cost of doing it this way is minimal.
                //...components
                ScreenUtil.setPriority(AbstractBComponent.class, formMap.getIntegerVal(ABSTRACT_BCOMPONENT_DL).intValue());
                ScreenUtil.setPriority(BAction.class, formMap.getIntegerVal(BACTION_DL).intValue());
                ScreenUtil.setPriority(BInput.class, formMap.getIntegerVal(BINPUT_DL).intValue());
                ScreenUtil.setPriority(BLink.class, formMap.getIntegerVal(BLINK_DL).intValue());
                ScreenUtil.setPriority(ScreenUtil.class, formMap.getIntegerVal(BLIST_DL).intValue());
                ScreenUtil.setPriority(ScreenUtil.class, formMap.getIntegerVal(BSELECT_DL).intValue());
                ScreenUtil.setPriority(ScreenUtil.class, formMap.getIntegerVal(BTABLE_DL).intValue());
                ScreenUtil.setPriority(ScreenUtil.class, formMap.getIntegerVal(BTEMPLATE_DL).intValue());
                ScreenUtil.setPriority(BText.class, formMap.getIntegerVal(BTEXT_DL).intValue());
                ScreenUtil.setPriority(BToggleButton.class, formMap.getIntegerVal(BTOGGLE_BUTTON_DL).intValue());
                ScreenUtil.setPriority(ComponentGateway.class, formMap.getIntegerVal(COMPONENT_GATEWAY_DL).intValue());
                //...models
                ScreenUtil.setPriority(DefaultListModel.class, formMap.getIntegerVal(DEFAULT_LIST_MODEL_DL).intValue());
                ScreenUtil.setPriority(DefaultPropertiesModel.class, formMap.getIntegerVal(DEFAULT_PROPERTIES_MODEL_DL).intValue());
                //...views
                ScreenUtil.setPriority(DefaultTableView.class, formMap.getIntegerVal(DEFAULT_TABLE_VIEW_DL).intValue());
                ScreenUtil.setPriority(DefaultTemplateView.class, formMap.getIntegerVal(DEFAULT_TEMPLATE_VIEW_DL).intValue());
                ScreenUtil.setPriority(DefaultView.class, formMap.getIntegerVal(DEFAULT_VIEW_DL).intValue());
                //...renderers
                ScreenUtil.setPriority(HTMLComponentRenderer.class, formMap.getIntegerVal(HTML_COMPONENT_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLComponentRenderer.class, formMap.getIntegerVal(HTML_COMPONENT_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLActionRenderer.class, formMap.getIntegerVal(HTML_ACTION_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLInputRenderer.class, formMap.getIntegerVal(HTML_INPUT_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLLinkRenderer.class, formMap.getIntegerVal(HTML_LINK_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLListRenderer.class, formMap.getIntegerVal(HTML_LIST_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLSelectRenderer.class, formMap.getIntegerVal(HTML_SELECT_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLTableRenderer.class, formMap.getIntegerVal(HTML_TABLE_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLToggleRenderer.class, formMap.getIntegerVal(HTML_TOGGLE_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLTemplateRenderer.class, formMap.getIntegerVal(HTML_TEMPLATE_RENDERER_DL).intValue());
                ScreenUtil.setPriority(HTMLTextRenderer.class, formMap.getIntegerVal(HTML_TEXT_RENDERER_DL).intValue());
                ScreenUtil.setPriority(TemplateHelper.class, formMap.getIntegerVal(TEMPLATE_HELPER_DL).intValue());
                ScreenUtil.setPriority(XMLComponentRenderer.class, formMap.getIntegerVal(XML_COMPONENT_RENDERER_DL).intValue());
                ScreenUtil.setPriority(XMLTemplateRenderer.class, formMap.getIntegerVal(XML_TEMPLATE_RENDERER_DL).intValue());
                ScreenUtil.setPriority(XMLTextRenderer.class, formMap.getIntegerVal(XML_TEXT_RENDERER_DL).intValue());

                //remember our success
                formMap.putState(SUCCESS_MESSAGE, new Boolean(true));

            } catch (ValidationException e) {
                ve = e;
            }

            //store the error message in the form and then put the form in the
            //the event context
            formMap.putState(ERROR_MESSAGE, ve);
            context.putState(FORM, formMap);

            //fire an update so that the screen will redraw
            ((CompModel) screen.compModel).fireModelChanged();

            //redirect to the get screen again
            throw new ClientSideRedirectException(new GetBConfig());
        }
    }

    //------------------------------------------------------------
    //                         Utility Methods
    //------------------------------------------------------------
    public TemplateModel getModel() {
        return new CompModel();
    }
}

