/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MasterConfig.java,v 1.9 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

//import org.apache.log4j.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.forms.validators.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.http.ServletUtil;
import org.enhydra.barracuda.core.util.http.SessionServices;
import org.enhydra.barracuda.core.util.l10n.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.util.srv.*;

import org.enhydra.barracuda.config.events.*;

/**
 * This class provides the methods needed to configure the util
 * screen
 */
public class MasterConfig extends DefaultEventGateway {

    protected static BCategory logger = BCategory.getBInstance(MasterConfig.class.getName());

    //form id (must be unique)
    public static final String FORM = MasterConfig.class.getName()+".Form";

    //model name
    public static final String MODEL_NAME = "Master";

    //model elements (these are the data items supported)
    public static final String LOCALE = "Locale";
    public static final String LOGGING = "Logging";


    //supported locales
    private static final Locale[] locales = new Locale[] {
                                        Locale.ENGLISH,    
                                        Locale.GERMAN,    
                                        new Locale("es",""),    
                                        new Locale("fr",""),    
                                        new Locale("sv",""),    
                                        new Locale("pt",""),    
                                        new Locale("fi",""),    
                                        }; 

    private static final String[] langs = new String[] {
                                        "English",    
                                        "Deutsch",    
                                        "Espaol",    
                                        "Franais",    
                                        "Svenska",    
                                        "Portugus",    
                                        "Suomi",    
                                        }; 

    //private vars
    private ListenerFactory updateConfigFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new UpdateConfigHandler();}        public String getListenerID() {return getID(UpdateConfigHandler.class);}};

    public MasterConfig() {
        //specify generic interest
        specifyLocalEventInterests(updateConfigFactory);
    }


    //------------------------------------------------------------
    //                       Data Models
    //------------------------------------------------------------
    /**
     * define the template model that backs this screen
     */
    class MasterModel extends AbstractTemplateModel {

        //register the model by name
        public String getName() {return MODEL_NAME;}
        
        //provide items by key
        public Object getItem(String key) {
            if (logger.isDebugEnabled()) logger.debug("Asking for key:"+key);    
            ViewContext vc = getViewContext();

            //Handle requests for data
            if (key.equals(LOCALE)) {
                DefaultListModel dlm = new DefaultListModel();
                Locale curloc = vc.getViewCapabilities().getClientLocale();
                int sel = Locales.findClosestLocale(curloc, locales, 0);
                for (int i=0, max=locales.length; i<max; i++) {
                    dlm.add(new DefaultItemMap(i, langs[i]));
                }
                BSelect bsComp = ScreenUtil.getSelectComp(vc, key, dlm, sel);
                bsComp.addEventListener(updateConfigFactory);
                return bsComp;
                
            } else if (key.equals(LOGGING)) {
                DefaultListModel dlm = new DefaultListModel();
                Locale curloc = vc.getViewCapabilities().getClientLocale();
                ResourceBundle rb = ResourceBundle.getBundle("org.enhydra.barracuda.config.xmlc.Config", curloc);
                dlm.add(new DefaultItemMap(1, Localize.getString(rb, "Config.General.Logging.Fatal")));
                dlm.add(new DefaultItemMap(2, Localize.getString(rb, "Config.General.Logging.Error")));
                dlm.add(new DefaultItemMap(3, Localize.getString(rb, "Config.General.Logging.Warn")));
                dlm.add(new DefaultItemMap(4, Localize.getString(rb, "Config.General.Logging.Info")));
                dlm.add(new DefaultItemMap(5, Localize.getString(rb, "Config.General.Logging.Debug")));
                int selIndex = ScreenUtil.cvtPriorityToInt(BCategory.getRoot().getPriority())-1;
                BSelect bsComp = ScreenUtil.getSelectComp(vc, key, dlm, selIndex);    //offset by 1 since option 0 ('System') is not a valid option here
                bsComp.addEventListener(updateConfigFactory);
                return bsComp;
            } else return super.getItem(key);
        }
    }

    //------------------------------------------------------------
    //                  Form Mappings, Validators
    //------------------------------------------------------------
    /**
     * define the form map that backs the model
     */
    class MasterForm extends DefaultFormMap {
        public MasterForm() {
            //define the elements (note: these don't need any validators). Note
            //also that we set the defaults to the current values. This allows
            //us just to reset all current values down below without checking to
            //see if the elements actually got passed in from the form.
            if (logger.isDebugEnabled()) logger.debug("Defining form elements");            
            this.defineElement(new DefaultFormElement(LOCALE, FormType.INTEGER, null, null, false));
            this.defineElement(new DefaultFormElement(LOGGING, FormType.INTEGER, new Integer(-1), null, false));
        }
    }

    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * UpdateConfigHandler - handle the request to update the config 
     * screen.
     */
    class UpdateConfigHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            if (logger.isDebugEnabled()) ServletUtil.showParams(context.getRequest(), logger);            
            
            //create the login form
            ValidationException ve = null;
            MasterForm formMap = new MasterForm();
            int sel = Locales.findClosestLocale(context.getViewCapabilities().getClientLocale(), locales, 0);
            MasterScreen screen = new MasterScreenFactory().getInstance(MasterConfig.this, context, locales[sel]);
            int newsel = sel;
            try {
                //map/validate the form
                formMap.map(context.getRequest()).validate(true);
                
                //update the language value if it changed
                newsel = formMap.getIntegerVal(LOCALE).intValue();
                if (newsel!=sel) {
                    Locales.saveClientLocale(context, locales[newsel]);
                    
                    //fire an update so that the screen will redraw
                    screen.bcRoot.validate();
                }
                
                //update the logging value if it changed
                int logPri = formMap.getIntegerVal(LOGGING).intValue();
                int curPri = ScreenUtil.cvtPriorityToInt(BCategory.getRoot().getPriority());
                if (logPri>-1 && logPri!=curPri) {
                    BCategory.getRoot().setPriority(ScreenUtil.cvtIntToPriority(logPri));
                    
                    //fire an update so that the screen will redraw
                    screen.bcRoot.validate();
                }

            } catch (ValidationException e) {
            }
            
            //redirect to the get screen again
            throw new ClientSideRedirectException(new GetBConfig());
        }
    }



    //------------------------------------------------------------
    //                         Utility Methods
    //------------------------------------------------------------
    public TemplateModel getModel() {
        return new MasterModel();
    }
}

