/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: MasterScreen.java,v 1.9 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import java.lang.ref.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.util.srv.*;
import org.enhydra.barracuda.core.view.*;

import org.enhydra.barracuda.config.events.*;
import org.enhydra.barracuda.config.xmlc.*;

/**
 * This class represents the master Barracuda config screen. It is stored 
 * in the users session on a per-locale basis, and is used kind of as a 
 * master repository for objects that make up the system.
 */
public class MasterScreen {

    //public constants
    protected static BCategory logger = BCategory.getBInstance(MasterScreen.class.getName());
    public static final String KEY = MasterScreen.class.getName()+".Key";

    public Document dom = null;
    public EventGateway egRoot = null;
    public BComponent bcRoot = null;
    public TabsModel tabsModel = null;
    public TemplateModel masterModel = null;
    public TemplateModel compModel = null;
    public TemplateModel dataModel = null;
    public TemplateModel domModel = null;
    public TemplateModel eventModel = null;
    public TemplateModel formsModel = null;
    public TemplateModel utilModel = null;
    public TemplateModel viewModel = null;
    public Locale locale = null;
    

    //------------------- MasterScreen ---------------------------
    public MasterScreen(EventGateway iegRoot, Locale ilocale) {
        if (logger.isDebugEnabled()) logger.debug("Creating new ConfigScreen object");
        
        //save a copy of the locale
        egRoot = iegRoot;
        locale = ilocale;
        
        //load the localized DOM template
        try {dom = DefaultDOMLoader.getGlobalInstance().getDOM(ConfigHTML.class, locale);}
        catch (IOException e) {logger.fatal ("Fatal Error loading DOM template:", e);}

        //create the root component
        bcRoot = new BComponent();

        //create the tab template
        BTemplate btTabs = new BTemplate();
        bcRoot.addChild(btTabs);
        //...create the views (we're storing directives in separate file)
        try {
            Properties props = new Properties();
            props.load(this.getClass().getResourceAsStream("Config.directives"));
            Node node = dom.getElementById("ConfigTabs");
            TemplateView tv = new DefaultTemplateView(node, "id", new MapStateMap(props));        
            btTabs.addView(tv);
        } catch (IOException e) {
            logger.fatal("Fatal err loading properties file:", e);
        }
        //...create the models
        //......Master model
        MasterConfig masterConfig = (MasterConfig) SimpleServiceFinder.findInstance(MasterConfig.class, egRoot, SimpleServiceFinder.DOWNSTREAM);
        if (masterConfig!=null) {
            masterModel = masterConfig.getModel();
            btTabs.addModel(masterModel);
        }
        //......Tabs model
        tabsModel = new TabsModel(btTabs);
        btTabs.addModel(tabsModel);
        //......Comp screen model
        CompConfig compConfig = (CompConfig) SimpleServiceFinder.findInstance(CompConfig.class, egRoot, SimpleServiceFinder.DOWNSTREAM);
        if (compConfig!=null) {
            compModel = compConfig.getModel();
            btTabs.addModel(compModel);
        }
        //......Data screen model
        DataConfig dataConfig = (DataConfig) SimpleServiceFinder.findInstance(DataConfig.class, egRoot, SimpleServiceFinder.DOWNSTREAM);
        if (dataConfig!=null) {
            dataModel = dataConfig.getModel();
            btTabs.addModel(dataModel);
        }
        //......Dom screen model
        DomConfig domConfig = (DomConfig) SimpleServiceFinder.findInstance(DomConfig.class, egRoot, SimpleServiceFinder.DOWNSTREAM);
        if (domConfig!=null) {
            domModel = domConfig.getModel();
            btTabs.addModel(domModel);
        }
        //......Event screen model
        EventConfig eventConfig = (EventConfig) SimpleServiceFinder.findInstance(EventConfig.class, egRoot, SimpleServiceFinder.DOWNSTREAM);
        if (eventConfig!=null) {
            eventModel = eventConfig.getModel();
            btTabs.addModel(eventModel);
        }
        //......Forms screen model
        FormsConfig formsConfig = (FormsConfig) SimpleServiceFinder.findInstance(FormsConfig.class, egRoot, SimpleServiceFinder.DOWNSTREAM);
        if (formsConfig!=null) {
            formsModel = formsConfig.getModel();
            btTabs.addModel(formsModel);
        }
        //......Util screen model
        UtilConfig utilConfig = (UtilConfig) SimpleServiceFinder.findInstance(UtilConfig.class, egRoot, SimpleServiceFinder.DOWNSTREAM);
        if (utilConfig!=null) {
            utilModel = utilConfig.getModel();
            btTabs.addModel(utilModel);
        }
        //......View screen model
        ViewConfig viewConfig = (ViewConfig) SimpleServiceFinder.findInstance(ViewConfig.class, egRoot, SimpleServiceFinder.DOWNSTREAM);
        if (viewConfig!=null) {
            viewModel = viewConfig.getModel();
            btTabs.addModel(viewModel);
        }
    }
}






