/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: UtilConfig.java,v 1.19 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.config;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.forms.validators.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.http.ServletUtil;
import org.enhydra.barracuda.core.util.http.SessionServices;
import org.enhydra.barracuda.core.util.l10n.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.util.srv.*;

import org.enhydra.barracuda.config.events.*;

/**
 * This class provides the methods needed to configure the util
 * screen
 */
public class UtilConfig extends DefaultEventGateway {

    protected static BCategory logger = BCategory.getBInstance(UtilConfig.class.getName());

    //form id (must be unique)
    public static final String FORM = UtilConfig.class.getName()+".Form";

    //model name
    public static final String MODEL_NAME = "Util";

    //model elements (these are the data items supported)
//    public static final String DEBUG_DL = "Debug_DebugLevel";
    public static final String LOCALES_DL = "Locales_DebugLevel";
//    public static final String SERVLET_UTIL_DL = "ServletUtil_DebugLevel";
    public static final String SESSION_SERVICES_DL = "SessionServices_DebugLevel";
//    public static final String SESSION_SERVICES_DEFAULT_TIMEOUT = "SessionServices_DefaultTimeout";
    public static final String SIMPLE_SERVICE_FINDER_DL = "SimpleServiceFinder_DebugLevel";
    public static final String ERROR_MESSAGE = "ErrorMessage";
    public static final String SUCCESS_MESSAGE = "SuccessMessage";
    public static final String UPDATE_BUTTON = "UpdateButton";

    //private vars
    private ListenerFactory updateConfigFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new UpdateConfigHandler();}        public String getListenerID() {return getID(UpdateConfigHandler.class);}};

    public UtilConfig() {
        //specify generic interest
        specifyLocalEventInterests(updateConfigFactory);
    }

    //------------------------------------------------------------
    //                       Data Models
    //------------------------------------------------------------
    /**
     * define the template model that backs this screen
     */
    class UtilModel extends AbstractTemplateModel {

        //register the model by name
        public String getName() {return MODEL_NAME;}
        
        //provide items by key
        public Object getItem(String key) {
            if (logger.isDebugEnabled()) logger.debug("Asking for key:"+key);    
            ViewContext vc = getViewContext();

            //Handle requests for data
            if (key.equals(LOCALES_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, Locales.class);
            } else if (key.equals(SESSION_SERVICES_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, SessionServices.class);
//            } else if (key.equals(SESSION_SERVICES_DEFAULT_TIMEOUT)) {
//                DefaultView dlv = new DefaultView(vc.getTemplateNode().cloneNode(true));
//                return new BInput(null, SESSION_SERVICES_DEFAULT_TIMEOUT, ""+SessionServices.DEFAULT_TIMEOUT, dlv, null);
            } else if (key.equals(SIMPLE_SERVICE_FINDER_DL)) {
                return ScreenUtil.getDebugLevelComp2(vc, key, SimpleServiceFinder.class);
            } else if (key.equals(ERROR_MESSAGE)) {
                return ScreenUtil.getErrMsg(vc, FORM, ERROR_MESSAGE);
            } else if (key.equals(SUCCESS_MESSAGE)) {
                return ScreenUtil.getSuccessMsg(vc, FORM, SUCCESS_MESSAGE);
            } else if (key.equals(UPDATE_BUTTON)) {
                return ScreenUtil.getUpdateButtonComp(vc, updateConfigFactory);
            } else return super.getItem(key);
        }
    }

    //------------------------------------------------------------
    //                  Form Mappings, Validators
    //------------------------------------------------------------
    /**
     * define the form map that backs the model
     */
    class UtilForm extends DefaultFormMap {
        public UtilForm() {
            //define the elements (note: these don't need any validators). Note
            //also that we set the defaults to the current values. This allows
            //us just to reset all current values down below without checking to
            //see if the elements actually got passed in from the form.
            if (logger.isDebugEnabled()) logger.debug("Defining form elements");            
            this.defineElement(new DefaultFormElement(LOCALES_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(Locales.class)), null, false));
            this.defineElement(new DefaultFormElement(SESSION_SERVICES_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(SessionServices.class)), null, false));
//            FormElement feTimeout = new DefaultFormElement(SESSION_SERVICES_DEFAULT_TIMEOUT, FormType.INTEGER, new Integer(SessionServices.DEFAULT_TIMEOUT), null, false);
//            feTimeout.setValidator(new ValidTypeValidator("Invalid Timeout value! You must enter an integer value."));
//            this.defineElement(feTimeout);
            this.defineElement(new DefaultFormElement(SIMPLE_SERVICE_FINDER_DL, FormType.INTEGER, new Integer(ScreenUtil.cvtPriorityToInt(SimpleServiceFinder.class)), null, false));
        }
    }

    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * UpdateConfigHandler - handle the request to update the config 
     * screen.
     */
    class UpdateConfigHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //figure out our target locale and get the appropriate screen
            Locale locale = context.getViewCapabilities().getClientLocale();
            MasterScreen screen = new MasterScreenFactory().getInstance(UtilConfig.this, context, locale);
            if (logger.isDebugEnabled()) ServletUtil.showParams(context.getRequest(), logger);            
            
            //create the login form
            ValidationException ve = null;
            UtilForm formMap = new UtilForm();
            try {
                //map/validate the form
                formMap.map(context.getRequest()).validate(true);
                
                //If there were no errors, update the data. Note that we just
                //assume that all the data is here. The reason we can do this is
                //because we prepopulated the form up above with the default values.
                //The more proper way would be to check each value, see if it's set,
                //and then only update it if the value has actually changed. Lot more
                //code to do that though, and since we're just setting statics, the
                //cost of doing it this way is minimal.
                ScreenUtil.setPriority(Locales.class, formMap.getIntegerVal(LOCALES_DL).intValue());
                ScreenUtil.setPriority(SessionServices.class, formMap.getIntegerVal(SESSION_SERVICES_DL).intValue());
//                SessionServices.DEFAULT_TIMEOUT = formMap.getIntegerVal(SESSION_SERVICES_DEFAULT_TIMEOUT).intValue();
                ScreenUtil.setPriority(SimpleServiceFinder.class, formMap.getIntegerVal(SIMPLE_SERVICE_FINDER_DL).intValue());

                //remember our success
                formMap.putState(SUCCESS_MESSAGE, new Boolean(true));

            } catch (ValidationException e) {
                ve = e;
            }

            //store the error message in the form and then put the form in the
            //the event context
            formMap.putState(ERROR_MESSAGE, ve);
            context.putState(FORM, formMap);

            //fire an update so that the screen will redraw
            ((UtilModel) screen.utilModel).fireModelChanged();

            //redirect to the get screen again
            throw new ClientSideRedirectException(new GetBConfig());
        }
    }

    //------------------------------------------------------------
    //                         Utility Methods
    //------------------------------------------------------------
    public TemplateModel getModel() {
        return new UtilModel();
    }
}

