/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: AbstractTemplateModel.java,v 1.6 2001/10/04 22:36:23 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;
import org.enhydra.barracuda.core.comp.model.*;

/**
 * This class provides the abstract implementation
 * for a Template Model.
 */
public abstract class AbstractTemplateModel implements TemplateModel {

    public static final String UNDEFINED = "Undefined";

    protected ViewContext viewContext = null;
    protected List listeners = new ArrayList();

    //--------------- AbstractTemplateModel ----------------------
    /**
     * Add a listener to the template that's notified each time a change
     * to the data model occurs.
     *
     * @param ml the TemplateModelListener
     */
    public void addModelListener(ModelListener ml) {
        listeners.add(ml);
    }

    /**
     * Remove a listener
     *
     * @param ml the TemplateModelListener
     */
    public void removeModelListener(ModelListener ml) {
        listeners.remove(ml);
    }
    
    /**
     * Forwards the given notification event to all
     * <code>TemplateModelListeners</code> that registered
     * themselves as listeners for this template model.
     *
     * @param e  the event to be forwarded
     *
     * @see #addTemplateModelListener
     * @see TemplateModelEvent
     */
    public void fireModelChanged() {
        Iterator it = listeners.iterator();
        ModelListener ml = null;
        while (it.hasNext()) {
            ml = (ModelListener) it.next();
            ml.modelChanged(this);
        }
    }


    //--------------- AbstractTemplateModel ----------------------
    /**
     * Add a listener to the template that's notified each time a change
     * to the data model occurs.
     *
     * @param l the TemplateModelListener
     */  
//    public void addTemplateModelListener(TemplateModelListener l) {
//        listeners.add(l);
//    }

    /**
     * Remove a listener from the template.
     *
     * @param l the TemplateModelListener
     */  
//    public void removeTemplateModelListener(TemplateModelListener l) {
//        listeners.remove(l);
//    }

    /**
     * Reset the model to its initial (unprocessed) state. This 
     * is a convenience method that gets invoked prior to the 
     * entire model being rendered. You only need to override this 
     * method if you want to do something (like reset internal counters)
     * before the model is queried
     */
/*
//csc_100401.1 - this doesn't need to be in here     
    public void resetModel() {
        //nop    
    }
*/
    /**
     * process any directives. return false to indicate a the node
     * containing this directive should be skipped.
     */
    public boolean processDirective(TemplateDirective td) {
        return true;    //by default, allow all directives
    }

    /**
     * get an item for a given key
     */
//    public Object getItem(String key, ViewContext vc) {
    public Object getItem(String key) {
        return getName()+"."+key+" "+UNDEFINED;
    }

    /**
     * Forwards the given notification event to all
     * <code>TemplateModelListeners</code> that registered
     * themselves as listeners for this template model.
     *
     * @param e  the event to be forwarded
     *
     * @see #addTemplateModelListener
     * @see TemplateModelEvent
     */
//    public void fireTemplateChanged(TemplateModelEvent e) {
//        Iterator it = listeners.iterator();
//        TemplateModelListener tml = null;
//        while (it.hasNext()) {
//            tml = (TemplateModelListener) it.next();
//            tml.modelChanged(e);
//        }
//    }

    /**
     * Return all the listeners for this given list model (returns 
     * a copy of the underlying List)
     *
     * @returns all the listeners for this given list model
     */
//    public List getListeners() { 
//        return new ArrayList(listeners); 
//    }

    //--------------- Contextual ---------------------------------
    /**
     * Specify the ViewContext. This method will generally be called
     * by the class that is using the model to actually render the data
     * in a view. The context will be specified prior to a render pass,
     * and the context will be reset to null after the render pass. 
     *
     * @param vc the current ViewContext
     */
    public void setViewContext(ViewContext ivc) {
        viewContext = ivc;
    }

    /**
     * Get the current ViewContext
     *
     * @return the current ViewContext
     */
    public ViewContext getViewContext() {
        return viewContext;
    }

}