/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: BScript.java,v 1.3 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.logging.*;


/**
 *
 */
public class BScript extends BComponent {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(BScript.class.getName());

    //private vars
    protected String src = null;

    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BScript() {}

    /**
     * Public constructor which creates the component and
     * binds it to a view. When rendered, it will make sure
     * that the src script is available to the client.
     *
     * <p>Null values may be passed in for any parameters,
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually
     * rendering the component
     *
     * @param src the src script that backs this component
     * @param view the View the component should be bound to
     */
    public BScript(String src, View view) {
        if (src!=null) this.setSrc(src);
        if (view!=null) this.addView(view);
    }



    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BScript.class, HTMLElement.class);
        installRendererFactory(rfHTML, BScript.class, HTMLDocument.class);
/*
        WMLRendererFactory rfWML = new WMLRendererFactory();
        installRendererFactory(rfWML, BScript.class, HTMLElement.class);
        installRendererFactory(rfWML, BScript.class, HTMLDocument.class);
*/
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLScriptRenderer();}
    }



    //--------------- BComponent ---------------------------------
    /**
     * Set the src for this particular component
     *
     * @param src the src script that backs this component
     */
    public void setSrc(String isrc) {
        src = isrc;
        invalidate();
    }

    /**
     * Get the src for this particular component
     *
     * @return the src for this particular component
     */
    public String getSrc() {
        return src;
    }

    /**
     * Render a specific view for the component.
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     * @param list a List of all the views for this component
     */
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info(depth, "rendering view: "+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
        } catch (DOMException e) {
            logger.warn(depth+2, "DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }

    /**
     * Get a String representation of the component
     */
    public String toString() {
        return src;
    }

}