/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BTable.java,v 1.8 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.DOMUtil;
import org.enhydra.barracuda.core.comp.model.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.comp.renderer.html.*;
import org.enhydra.barracuda.core.comp.renderer.xml.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 *
 * Q's:
 */
public class BTable extends BComponent {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(BTable.class.getName());
    
    //private vars


    /**
     * @clientCardinality 1 
     */
    protected TableModel model = null;

    /**
     * @clientCardinality 1 
     */
    protected TableModel headerModel = null;

    /**
     * @clientCardinality 1 
     */
    protected TableModel footerModel = null;
    private LocalModelListener callback = null;
    private LocalModelListener headerCallback = null;
    private LocalModelListener footerCallback = null;
    protected Node templateNode = null;


    //--------------- Constructors -------------------------------
    /**
     * Public noargs constructor
     */
    public BTable() {
        this (null, null);
    }
    
    /**
     * Public constructor which creates the component and
     * binds it to a specific model. The component is also
     * bound to the specified view.
     *
     * <p>Null values may be passed in for any parameters, 
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually 
     * rendering the component
     *
     * @param model the specific model to back this component
     * @param view the View the component should be bound to
     */
    public BTable(TableModel imodel, TableView view) {
        this (null, imodel, null, view);
    }
    
    /**
     * Public constructor which creates the component and
     * binds it to a specific model. The component is also
     * bound to the specified view.
     *
     * <p>Null values may be passed in for any parameters, 
     * but if you do so you will need manually provide these
     * values (via the accessor methods) prior to actually 
     * rendering the component
     *
     * @param headerModel the specific header model to back this component
     * @param bodyModel the main data model that backs this component
     * @param footerModel the specific footer model to back this component
     * @param view the View the component should be bound to
     */
    public BTable(TableModel iheaderModel, TableModel ibodyModel, TableModel ifooterModel, TableView view) {
        if (iheaderModel!=null) setHeaderModel(iheaderModel);
        if (ibodyModel!=null) setModel(ibodyModel);
        if (ifooterModel!=null) setFooterModel(ifooterModel);
        if (view!=null) this.addView(view);    
    }
    




    //--------------- Renderer -----------------------------------
    /**
     * Default component renderer factory registrations
     */
    static {
        HTMLRendererFactory rfHTML = new HTMLRendererFactory();
        installRendererFactory(rfHTML, BTable.class, HTMLElement.class);
        installRendererFactory(rfHTML, BTable.class, HTMLDocument.class);
    }

    /**
     * HTML RendererFactory
     */
    static class HTMLRendererFactory implements RendererFactory {
        public Renderer getInstance() {return new HTMLTableRenderer();}
    }




    //--------------- BTable ------------------------------
    /**
     * Set the model that backs the table. This causes the table to register
     * as a listener on the model, so any changes to it will be reflected in 
     * the table.
     *
     * @param model the model that backs the table
     */
    public void setModel(TableModel imodel) {

        //deregister if possible
        if (model!=null && callback!=null) {
//            model.removeTableModelListener(callback);
            model.removeModelListener(callback);
        }
    
        //set the model    
        model = imodel;
        invalidate();
        
        //reregister if possible
        if (model!=null) {
            if (callback==null) callback = new LocalModelListener();
//            model.addTableModelListener(callback);
            model.addModelListener(callback);
        }
    }
    
    /**
     * Get the model that backs the table
     *
     * @return the model that backs the table
     */
    public TableModel getModel() {
        return model;    
    }
    
    /**
     * Set the header model that backs the table. This causes 
     * the table to register as a listener on the model, so 
     * any changes to it will be reflected in the table header.
     *
     * @param model the headermodel that backs the table
     */
    public void setHeaderModel(TableModel imodel) {

        //deregister if possible
        if (headerModel!=null && headerCallback!=null) {
//            headerModel.removeTableModelListener(headerCallback);
            headerModel.removeModelListener(headerCallback);
        }
    
        //set the model    
        headerModel = imodel;
        invalidate();
        
        //reregister if possible
        if (headerModel!=null) {
            if (headerCallback==null) headerCallback = new LocalModelListener();
//            headerModel.addTableModelListener(headerCallback);
            headerModel.addModelListener(headerCallback);
        }
    }
    
    /**
     * Get the header model that backs the table
     *
     * @return the header model that backs the table
     */
    public TableModel getHeaderModel() {
        return headerModel;    
    }
    
    /**
     * Set the footer model that backs the table. This causes 
     * the table to register as a listener on the model, so 
     * any changes to it will be reflected in the table footer.
     *
     * @param model the footer model that backs the table
     */
    public void setFooterModel(TableModel imodel) {

        //deregister if possible
        if (footerModel!=null && footerCallback!=null) {
//            footerModel.removeTableModelListener(footerCallback);
            footerModel.removeModelListener(footerCallback);
        }
    
        //set the model    
        footerModel = imodel;
        invalidate();
        
        //reregister if possible
        if (footerModel!=null) {
            if (footerCallback==null) footerCallback = new LocalModelListener();
//            footerModel.addTableModelListener(footerCallback);
            footerModel.addModelListener(footerCallback);
        }
    }
    
    /**
     * Get the footer model that backs the table
     *
     * @return the footer model that backs the table
     */
    public TableModel getFooterModel() {
        return footerModel;    
    }
    
    /**
     * Render a specific view for the component. 
     *
     * @param view View to be rendered
     * @param vc ViewContext for the client view
     * @throws RenderException if the particular View is not supported
     * @param list a List of all the views for this component
     */
    protected void renderView (View view, ViewContext vc, int depth) throws RenderException {
        if (logger.isInfoEnabled()) logger.info("rendering view: "+view);

        //actually render the view according to known interfaces
        try {
            Renderer r = getRenderer(view);
            r.renderComponent(this, view, vc);
            
        } catch (DOMException e) {
            logger.warn(1, "DOM Error:", e);
            throw new DOMAccessException("Error rendering component in view:"+e, e);
        }
    }




    //--------------- Lifecycle ----------------------------------
    /**
     * Destroy cycle. The component should use this cycle to
     * perform any special cleanup.
     */
    public void destroyCycle() {
        //default destroy
        super.destroyCycle();
    
        //we set the model to null so that the component can be 
        //garbage collected. If we don't do this, the model retains
        //a reference back to the component and so the component 
        //will never be freed up...
        setModel(null);        
        setHeaderModel(null);        
        setFooterModel(null);        
    }




    //--------------- Utility ------------------------------------
/*    
    class LocalTableModelListener implements TableModelListener {
        //for right now, just invalidate everything. In the future,
        //we could get a lot smarter and keep track of the ranges
        //changed and then only redraw those. This would greatly 
        //speed rendering, because we'd only need to redraw the parts
        //of the DOM that changed...
        public void tableChanged(TableModelEvent e) {
            invalidate();
        }
    }
*/    
    class LocalModelListener implements ModelListener {
        //get notified when one of the underlying models changes
        public void modelChanged(Model m) {
            invalidate();
        }
    }
}