/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultListModel.java,v 1.7 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;

import org.enhydra.barracuda.core.util.logging.*;

/**
 * This class provides a default implementation for List Model.
 * This implementation is backed by a java.util.ArrayList.
 */
public class DefaultListModel extends AbstractListModel implements List {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(DefaultListModel.class.getName());

    protected List items = new ArrayList();

    //--------------- ListModel ----------------------------------
    /**
     * Get an element at a specific index
     *
     * @param index the target index
     * @param vc the ViewContext
     * @return the element at the specific index
     */
//    public Object getItemAt(int index, ViewContext vc) {
    public Object getItemAt(int index) {
        return get(index);
    }

    /**
     * Get the size of the list
     *
     * @return the size of the list
     * @see List#size()
     */
    public int getSize() {
        return size();
    }




    //--------------- List ---------------------------------------
    /**
     * Returns the number of elements in this list.  
     *
     * @return the number of elements in this list.
     * @see List#size()
     */
    public int size() {
        return items.size();
    }

    /**
     * Returns <tt>true</tt> if this list contains no elements.
     *
     * @return <tt>true</tt> if this list contains no elements.
     * @see List#isEmpty()
     */
    public boolean isEmpty() {
        return items.isEmpty();
    }

    /**
     * 
     * Returns <tt>true</tt> if this list contains the specified element.
     * 
     * @param o element whose presence in this list is to be tested.
     * @return <tt>true</tt> if this list contains the specified element.
     * @see List#contains(Object)
     */
    public boolean contains(Object o) {
        return items.contains(o);
    }

    /**
     * Returns an iterator over the elements in this list in proper sequence.
     *
     * @return an iterator over the elements in this list in proper sequence.
     * @see List#iterator()
     */
    public Iterator iterator() {
        return items.iterator();
    }

    /**
     * Returns an array containing all of the elements in this list in proper
     * sequence.  Obeys the general contract of the
     * <tt>Collection.toArray</tt> method.
     *
     * @return an array containing all of the elements in this list in proper
     *           sequence.
     * @see List#toArray()
     */
    public Object[] toArray() {
        return items.toArray();
    }

    /**
     * Returns an array containing all of the elements in this list in proper
     * sequence; the runtime type of the returned array is that of the
     * specified array.  Obeys the general contract of the
     * <tt>Collection.toArray(Object[])</tt> method.
     *
     * @param a the array into which the elements of this list are to
     *        be stored, if it is big enough; otherwise, a new array of the
     *         same runtime type is allocated for this purpose.
     * @return  an array containing the elements of this list.
     * @throws ArrayStoreException if the runtime type of the specified array
     *           is not a supertype of the runtime type of every element in
     *           this list.
     * @see List#toArray(Object[])
     */
    public Object[] toArray(Object a[]) {
        return items.toArray(a);
    }

    /**
     * Appends the specified element to the end of this list (optional
     * operation). <p>
     *
     * @param o element to be appended to this list.
     * @return <tt>true</tt> (as per the general contract of the
     *            <tt>Collection.add</tt> method).
     * @throws UnsupportedOperationException if the <tt>add</tt> method is not
     *           supported by this list.
     * @throws ClassCastException if the class of the specified element
     *           prevents it from being added to this list.
     * @throws IllegalArgumentException if some aspect of this element
     *            prevents it from being added to this collection.
     * @see List#add(Object)
     */
    public boolean add(Object o) {
        boolean result = items.add(o);
        if (result) {
//            int index = items.size()-1;
//            fireIntervalAdded(this, index, index);
            fireModelChanged();
        }
        return result;
    }

    /**
     * Removes the first occurrence in this list of the specified element 
     * (optional operation).  If this list does not contain the element, it is
     * unchanged.  
     *
     * @param o element to be removed from this list, if present.
     * @return <tt>true</tt> if this list contained the specified element.
     * @throws UnsupportedOperationException if the <tt>remove</tt> method is
     *          not supported by this list.
     * @see List#remove(Object)
     */
    public boolean remove(Object o) {
        int pos = items.indexOf(o);
        boolean result = items.remove(o);
//        if (result) fireIntervalRemoved(this, pos, pos);
        if (result) fireModelChanged();
        return result;
    }

    /**
     * 
     * Returns <tt>true</tt> if this list contains all of the elements of the
     * specified collection.
     *
     * @param c collection to be checked for containment in this list.
     * @return <tt>true</tt> if this list contains all of the elements of the
     *            specified collection.
     * @see List#containsAll(Collection)
     */
    public boolean containsAll(Collection c) {
        return items.containsAll(c);
    }

    /**
     * Appends all of the elements in the specified collection to the end of
     * this list, in the order that they are returned by the specified
     * collection's iterator (optional operation).  
     *
     * @param c collection whose elements are to be added to this list.
     * @return <tt>true</tt> if this list changed as a result of the call.
     * @throws UnsupportedOperationException if the <tt>addAll</tt> method is
     *         not supported by this list.
     * @throws ClassCastException if the class of an element in the specified
     *            collection prevents it from being added to this list.
     * @throws IllegalArgumentException if some aspect of an element in the
     *         specified collection prevents it from being added to this
     *         list.
     * @see List#addAll(Collection)
     */
    public boolean addAll(Collection c) {
        int start = items.size()-1;
        boolean result = items.addAll(c);
//        if (result) fireIntervalAdded(this, start, items.size()-1);
        if (result) fireModelChanged();
        return result;
    }

    /**
     * Inserts all of the elements in the specified collection into this
     * list at the specified position (optional operation).  Shifts the
     * element currently at that position (if any) and any subsequent
     * elements to the right (increases their indices).  
     *
     * @param index index at which to insert first element from the specified
     *                collection.
     * @param c elements to be inserted into this list.
     * @return <tt>true</tt> if this list changed as a result of the call.
     * @throws UnsupportedOperationException if the <tt>addAll</tt> method is
     *          not supported by this list.
     * @throws ClassCastException if the class of one of elements of the
     *           specified collection prevents it from being added to this
     *           list.
     * @throws IllegalArgumentException if some aspect of one of elements of
     *          the specified collection prevents it from being added to
     *          this list.
     * @throws IndexOutOfBoundsException if the index is out of range (index
     *          &lt; 0 || index &gt; size()).
     * @see List#addAll(int,Collection)
     */
    public boolean addAll(int index, Collection c) {
        boolean result = items.addAll(index, c);
//        if (result) fireIntervalAdded(this, index, items.size()-1);
        if (result) fireModelChanged();
        return result;
    }

    /**
     * Removes from this list all the elements that are contained in the
     * specified collection (optional operation).
     *
     * @param c collection that defines which elements will be removed from
     *          this list.
     * @return <tt>true</tt> if this list changed as a result of the call.
     * @throws UnsupportedOperationException if the <tt>removeAll</tt> method
     *           is not supported by this list.
     * @see List#removeAll(Collection)
     */
    public boolean removeAll(Collection c) {
        boolean result = items.removeAll(c);
//        if (result) fireContentsChanged(this, 0, items.size()-1);
        if (result) fireModelChanged();
        return result;
    }

    /**
     * Retains only the elements in this list that are contained in the
     * specified collection (optional operation).  In other words, removes
     * from this list all the elements that are not contained in the specified
     * collection.
     *
     * @param c collection that defines which elements this set will retain.
     * @return <tt>true</tt> if this list changed as a result of the call.
     * @throws UnsupportedOperationException if the <tt>retainAll</tt> method
     *           is not supported by this list.
     * 
     * @see List#retainAll(Collection)
     */
    public boolean retainAll(Collection c) {
        boolean result = items.retainAll(c);
//        if (result) fireContentsChanged(this, 0, items.size()-1);
        if (result) fireModelChanged();
        return result;
    }

    /**
     * Removes all of the elements from this list
     *
     * @throws UnsupportedOperationException if the <tt>clear</tt> method is
     *           not supported by this list.
     * @see List#clear()
     */
    public void clear() {
        items.clear();
    }

    /**
     * Compares the specified object with this list for equality.  Returns
     * <tt>true</tt> if and only if the specified object is also a list, both
     * lists have the same size, and all corresponding pairs of elements in
     * the two lists are <i>equal</i>.  
     *
     * @param o the object to be compared for equality with this list.
     * @return <tt>true</tt> if the specified object is equal to this list.
     * @see List#equals(Object)
     */
    public boolean equals(Object o) {
        return items.equals(o);
    }

    /**
     * Returns the hash code value for this list.
     *
     * @return the hash code value for this list.
     * @see List#hashCode()
     */
    public int hashCode() {
        return items.hashCode();
    }

    /**
     * Returns the element at the specified position in this list.
     *
     * @param index index of element to return.
     * @return the element at the specified position in this list.
     * @throws IndexOutOfBoundsException if the index is out of range (index
     *           &lt; 0 || index &gt;= size()).
     * @see List#get(int)
     */
    public Object get(int index) {
        return items.get(index);
    }

    /**
     * Replaces the element at the specified position in this list with the
     * specified element (optional operation).
     *
     * @param index index of element to replace.
     * @param element element to be stored at the specified position.
     * @return the element previously at the specified position.
     * @throws UnsupportedOperationException if the <tt>set</tt> method is not
     *          supported by this list.
     * @throws    ClassCastException if the class of the specified element
     *           prevents it from being added to this list.
     * @throws    IllegalArgumentException if some aspect of the specified
     *          element prevents it from being added to this list.
     * @throws    IndexOutOfBoundsException if the index is out of range
     *          (index &lt; 0 || index &gt;= size()).  
     * @see List#set(int,Object)
     */
    public Object set(int index, Object element) {
        Object result = items.set(index, element);
//        if (result!=null) fireContentsChanged(this, index, index);
        if (result!=null) fireModelChanged();
        return result;
    }

    /**
     * Inserts the specified element at the specified position in this list
     * (optional operation).  Shifts the element currently at that position
     * (if any) and any subsequent elements to the right (adds one to their
     * indices).
     *
     * @param index index at which the specified element is to be inserted.
     * @param element element to be inserted.
     * @throws UnsupportedOperationException if the <tt>add</tt> method is not
     *          supported by this list.
     * @throws    ClassCastException if the class of the specified element
     *           prevents it from being added to this list.
     * @throws    IllegalArgumentException if some aspect of the specified
     *          element prevents it from being added to this list.
     * @throws    IndexOutOfBoundsException if the index is out of range
     *          (index &lt; 0 || index &gt; size()).
     * @see List#add(int,Object)
     */
    public void add(int index, Object element) {
        items.add(index, element);
//        fireIntervalAdded(this, index, index);
        fireModelChanged();
    }

    /**
     * Removes the element at the specified position in this list (optional
     * operation).  Shifts any subsequent elements to the left (subtracts one
     * from their indices).  Returns the element that was removed from the
     * list.
     *
     * @param index the index of the element to removed.
     * @return the element previously at the specified position.
     * @throws UnsupportedOperationException if the <tt>remove</tt> method is
     *          not supported by this list.
     * @throws IndexOutOfBoundsException if the index is out of range (index
     *            &lt; 0 || index &gt;= size()).
     * @see List#remove(int)
     */
    public Object remove(int index) {
        Object result = items.remove(index);
//        if (result!=null) fireIntervalRemoved(this, index, index);
        if (result!=null) fireModelChanged();
        return result;
    }

    /**
     * Returns the index in this list of the first occurrence of the specified
     * element, or -1 if this list does not contain this element.
     *
     * @param o element to search for.
     * @return the index in this list of the first occurrence of the specified
     *            element, or -1 if this list does not contain this element.
     * @see List#indexOf(Object)
     */
    public int indexOf(Object o) {
        return items.indexOf(o);
    }

    /**
     * Returns the index in this list of the last occurrence of the specified
     * element, or -1 if this list does not contain this element.
     *
     * @param o element to search for.
     * @return the index in this list of the last occurrence of the specified
     *            element, or -1 if this list does not contain this element.
     * @see List#lastIndexOf(Object)
     */
    public int lastIndexOf(Object o) {
        return items.lastIndexOf(o);
    }

    /**
     * Returns a list iterator of the elements in this list (in proper
     * sequence).
     *
     * @return a list iterator of the elements in this list (in proper
     *            sequence).
     * @see List#listIterator()
     */
    public ListIterator listIterator() {
        return items.listIterator();
    }

    /**
     * Returns a list iterator of the elements in this list (in proper
     * sequence), starting at the specified position in this list.
     *
     * @param index index of first element to be returned from the
     *            list iterator (by a call to the <tt>next</tt> method).
     * @return a list iterator of the elements in this list (in proper
     *            sequence), starting at the specified position in this list.
     * @throws IndexOutOfBoundsException if the index is out of range (index
     *         &lt; 0 || index &gt; size()).
     * @see List#listIterator(int)
     */
    public ListIterator listIterator(int index) {
        return items.listIterator(index);
    }

    /**
     * Returns a view of the portion of this list between the specified
     * <tt>fromIndex</tt>, inclusive, and <tt>toIndex</tt>, exclusive.  (If
     * <tt>fromIndex</tt> and <tt>toIndex</tt> are equal, the returned list is
     * empty.)  The returned list is backed by this list, so changes in the
     * returned list are reflected in this list, and vice-versa.  The returned
     * list supports all of the optional list operations supported by this
     * list.<p>
     *
     * @param fromIndex low endpoint (inclusive) of the subList.
     * @param toIndex high endpoint (exclusive) of the subList.
     * @return a view of the specified range within this list.
     * @throws IndexOutOfBoundsException for an illegal endpoint index value
     *     (fromIndex &lt; 0 || toIndex &gt; size || fromIndex &gt; toIndex).
     * @see List#subList(int,int)
     */
    public List subList(int fromIndex, int toIndex) {
        return items.subList(fromIndex, toIndex);
    }




    //--------------- Object -------------------------------------
    /**
     * Create a string representation of the list and return it.
     * This method is invoked by the list component when rendering 
     * a list into a text based item that does not inherantly support 
     * a list structure
     *
     * @return a string representation of the list
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(500);
        String sep = "";
        Iterator it = items.iterator();
        while (it.hasNext()) {
            sb.append(sep+it.next().toString());
            sep = ", ";
        }
        String result = sb.toString().trim();
        if (result==null || result.length()<1) result = "[empty list]";
        return result;
    }

}