/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultPropertiesModel.java,v 1.8 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;

import org.enhydra.barracuda.core.util.l10n.*;
import org.enhydra.barracuda.core.util.logging.*;


/**
 * This class provides a default implementation of a TemplateModel
 * that looks up its keys in a properties file
 */
public class DefaultPropertiesModel extends AbstractTemplateModel {

    //public vars
    protected static BCategory logger = BCategory.getBInstance(DefaultPropertiesModel.class.getName());

    private final String DEFAULT_RESOURCE = "DefaultResource";

    protected String modelName = null;
    protected String propFileName = null;


    //--------------- Constructors ---------------------------
    public DefaultPropertiesModel() {
        super();
    }
    
    public DefaultPropertiesModel(String ipropFileName) {
        setPropFileName(ipropFileName);
    }

    
    //--------------- DefaultPropertiesModel -----------------
    public void setPropFileName(String ipropFileName) {
        //set the prop file name
        propFileName = ipropFileName;

        //set the model name based on the prop file name. While the 
        //prop file name is fully qualified, the model name will just be
        //set to the last portion of the prop file name
        if (propFileName!=null) {
            int pos = propFileName.lastIndexOf(".");
            if (pos>-1) modelName = propFileName.substring(pos+1);
            else modelName = propFileName;
        }
    }
    
    

    //--------------- AbstractTemplateModel ------------------
    public String getName() {
        return modelName;
    }

//    public Object getItem(String key, ViewContext vc) {
    public Object getItem(String key) {
        //By default, we're going to try and return the key value from
        //the underlying properties file. To do this, we first need to
        //try and figure out what format our view expects. We get this 
        //information by looking in the ViewCapabilities file. If the
        //VC info is not set, just assume default locale...

        //figure out the target locale
        ViewContext vc = getViewContext();
        Locale targetLocale = vc.getViewCapabilities().getClientLocale();
    
        //get the appropriate resource bundle (we don't need to worry about
        //caching the resource bundle since the ResourceBundle class already
        //does that for us (cool!))
        ResourceBundle rb = ResourceBundle.getBundle(propFileName, targetLocale);
        
        //now try looking up the value
        return Localize.getString(rb, key);
    }
}