/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HTMLScriptRenderer.java,v 1.4 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;

/**
 * This class handles the default rendering script references into an HTML view.
 */
public class HTMLScriptRenderer extends DOMComponentRenderer {

    protected static BCategory logger = BCategory.getBInstance(HTMLScriptRenderer.class.getName());

    protected EnabledHelper eh = new EnabledHelper();
    
    /**
     * 
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //unlike other components, this one really doesn't care what we're bound
        //to...we do need to make sure the component is a script component
        if (!(comp instanceof BScript)) throw new NoSuitableRendererException("This renderer can only render BScript components");
        BScript bsComp = (BScript) comp;
        Node node = view.getNode();

        //if the src attribute is null don't render this component
        String src = bsComp.getSrc();

        //if the client doesn't support scripting then bail
        ViewCapabilities vcap = vc.getViewCapabilities();
        if (!(vcap.getScriptingType() instanceof ScriptingType.JavaScript1x)) return;

        //now allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //find the root
        Document doc = node.getOwnerDocument();
        Element elRoot = doc.getDocumentElement();
        
        //now see if there is a head element
        Element elHead = null;
        Node child = elRoot.getFirstChild();
        while (child!=null) {
            if (child instanceof HTMLHeadElement) {
                elHead = (HTMLHeadElement) child;
                break;
            }    
            child = child.getNextSibling();
        }
        
        //if not, create one and add it in
        if (elHead==null) {
            elHead = doc.createElement("HEAD");
            elRoot.insertBefore(elHead, elRoot.getFirstChild());
        }

        //now see if we have script elements that match already in place
        //(if so, we don't need to do anything)
        HTMLScriptElement elScript = null;
        child = elHead.getFirstChild();
        while (child!=null) {
            if (child instanceof HTMLScriptElement) {
                HTMLScriptElement scel = (HTMLScriptElement) child;
                if (src.equals(scel.getSrc())) {
                    elScript = scel;
                    break;
                }
            }    
            child = child.getNextSibling();
        }
        
        //if we don't, create one
        if (elScript==null) {
            elScript = (HTMLScriptElement) doc.createElement("SCRIPT");
            elHead.appendChild(elScript);
            elScript.setType("text/javascript");
            elScript.setSrc(src);
        }
  
//for script components, I don't think we need to worry about whether they're enabled or not        
        //finally, make sure we reflect the components enabled/disabled status
//        eh.setEnabled(node, comp.isEnabled());
    }
    
}