/**
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: NamingHelper.java,v 1.1 2001/10/02 22:24:58 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.util.logging.*;

/**
 * Simple helper class to assist with determining an elements name. 
 * Created csc_100201.2
 */
public class NamingHelper {

    protected static BCategory logger = BCategory.getBInstance(NamingHelper.class.getName());
    
    /**
     * Simple helper method to get a name for an element.
     * If the name is not set, a unique name will be generated,
     * and the name attribute in the element will be set accordingly.
     * Only works for HTML elements that actually support the name
     * attribute (otherwise just returns "")
     *
     * @param el a target element
     * @returns the name of that element
     */
    public static String getName(Element el) {
        String name = "";
        if (el instanceof HTMLAnchorElement ||
            el instanceof HTMLAppletElement ||
            el instanceof HTMLButtonElement ||
            el instanceof HTMLFormElement ||
            el instanceof HTMLFrameElement ||
            el instanceof HTMLIFrameElement ||
            el instanceof HTMLInputElement ||
            el instanceof HTMLMapElement ||
            el instanceof HTMLMetaElement ||
            el instanceof HTMLObjectElement ||
            el instanceof HTMLParamElement ||
            el instanceof HTMLSelectElement ||
            el instanceof HTMLTextAreaElement) {

            name = el.getAttribute("name");
            if (name==null || name.equals("")) {
                name = "el_"+new Object().hashCode();
                el.setAttribute("name", name);
            }
        }
        return name;
    }
    
}
