/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XMLTemplateRenderer.java,v 1.6 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.xml;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * This class handles the default rendering of text into an XML view.
 */
public class XMLTemplateRenderer extends XMLComponentRenderer {

    protected static BCategory logger = BCategory.getBInstance(XMLTemplateRenderer.class.getName());
    
    protected TemplateHelper th = null;
    
    /**
     *
     */
    public XMLTemplateRenderer() {
        this(null);    
    }
    
    /**
     *
     */
    public XMLTemplateRenderer(TemplateHelper ith) {
        setTemplateHelper(ith);
    }
    
    /**
     *
     */
    public void setTemplateHelper(TemplateHelper ith) {
        th = ith;
    }
    
    /**
     *
     */
    public TemplateHelper getTemplateHelper() {
        if (th==null) th = new TemplateHelper(this);
        return th;
    }
    
    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //make sure the component is a template component
        if (!(comp instanceof BTemplate)) throw new NoSuitableRendererException("This renderer can only render BTemplate components");
        BTemplate btemplate = (BTemplate) comp;
        Node node = view.getNode();

        //make sure the View implements TemplateView
        if (!(view instanceof TemplateView)) throw new InvalidViewException ("Component is bound to an unsupported View:"+view);
        TemplateView tview = (TemplateView) view;

        //show what we're doing
        if (logger.isDebugEnabled()) showNodeInterfaces(view, logger);

        //first, allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        //now delegate the actual rendering to the TemplateHelper
        getTemplateHelper().render(btemplate, tview, vc);
    }
    
}