/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: BaseEvent.java,v 1.6 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

import org.enhydra.barracuda.core.util.data.StateMap;

/**
 * This interface defines the methods needed to implement
 * a BaseEvent
 */
public interface BaseEvent extends StateMap, Cloneable {
    public static String EVENT_ID = "eid$";        //useful constant

    /**
     * set the source for an event
     *
     * @param isource the source for this event
     */
    public void setSource(Object isource);
    
    /**
     * get the event source (may be null)
     *
     * @return the source for this event
     */
    public Object getSource();

    /**
     * set the event extension
     *
     * @param iext the target event extension
     */    
    public void setEventExtension(String iext);

    /**
     * get the event extension
     *
     * @return the target event extension
     */    
    public String getEventExtension();
    
    /**
     * mark the event as handled/unhandled
     *
     * @param true if the event is handled
     */    
    public void setHandled(boolean val);
    
    /**
     * get the handled status for the event
     *
     * @return true if the event is handled
     */    
    public boolean isHandled();

    /**
     * Add a specific listener id this event should be delivered to.
     * Events can be targeted to more than one ID.
     *
     * @param id the Listener ID the event should target
     */        
    public void addListenerID(String id);

    /**
     * Get the list of id's this event is specifically targeted for.
     * May return null if there are none.
     *
     * @return a List of ID's this event is specifically targeting
     */        
    public List getListenerIDs();

    /**
     * Get the ID that identifies this event. This will typically be the
     * class name.
     *
     * @return a string that uniquely identifies this event
     */
    public String getEventID();

    // RTL20010719
    /**
     * Get the id + extension of this event.
     *
     * @return the id and extension of this event
     */
    public String getEventIDWithExtension();
    // end RTL20010719

    /**
     * Get the timestamp
     *
     * @return the last time this event was touched
     */
    public long getTimestamp();

    /**
     * Update the timestamp on the event
     */
    public void touch();

    /**
     * Reset the event to it's default state
     */    
    public void reset();
}
