/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: InterruptDispatchException.java,v 1.4 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

/**
 * This class defines a InterruptDispatchException...it is used to
 * interrupt the dispatch, flag all events currently in the queue 
 * as handled, and then redispatch the new event
 */
public class InterruptDispatchException extends EventException {
    /**
     * @clientCardinality 1 
     */
    private BaseEvent newEvent = null;

    /**
     * The public contructor for InterruptDispatchException
     *
     * @param inewEvent the event we want to dispatch instead
     */
    public InterruptDispatchException (BaseEvent inewEvent) {
        this(null, inewEvent);
    }

    /**
     * The public contructor for InterruptDispatchException
     *
     * @param s a String describing the exception
     * @param inewEvent the event we want to dispatch instead
     */
    public InterruptDispatchException (String s, BaseEvent inewEvent) {
        this(s, inewEvent, null);
    }

    /**
     * The public contructor for InterruptDispatchException
     *
     * @param s a String describing the exception
     * @param inewEvent the event we want to dispatch instead
     * @param ibaseException the root exception behind this error
     */
    public InterruptDispatchException (String s, BaseEvent inewEvent, Exception ibaseException) {
        super(s, ibaseException);
        newEvent = inewEvent;
    }
    
    /**
     * Get the new event that triggered this interrupt
     *
     * @return the event we wish to fire instead
     */
    public BaseEvent getNewEvent() {
        return newEvent;
    }
}
