/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BTemplateViewHandler.java,v 1.3 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event.helper;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * <p>A custom implementation of the default view handler tailored for
 * template components. All you have to do is instantiate the class
 * with the name of the template and a reference to the model; it takes 
 * care of everything else.
 */
public abstract class BTemplateViewHandler extends DefaultViewHandler {

    //public vars...eventually, these should probably be final
    protected static BCategory logger = BCategory.getBInstance(BTemplateViewHandler.class.getName());


    //-------------------- DefaultViewHandler --------------------
    /**
     * Return an instance of the template model
     */
    public abstract TemplateModel getTemplateModel();
    
    /**
     * Return an instance of the template class (must implement Document)
     */
    public abstract Class getTemplateClass();
    
    /**
     * Generate the view
     */
    public Document handleViewEvent(BComponent root, ViewEventContext vec) throws EventException, ServletException, IOException {
        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass());

        //create a template component and bind it to the views
        TemplateView tv = new DefaultTemplateView(page.getDocumentElement());
        BTemplate templateComp = new BTemplate(getTemplateModel(), tv);
        root.addChild(templateComp);

        //return the page
        return page;
    }
}
