/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EventRedirectFactory.java,v 1.5 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event.helper;

import java.io.*;
import java.util.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * This class provides a simple factory that will handle
 * events by simply throwing a client side redirect, in
 * effect acting as an event forwarder. 
 */
public class EventRedirectFactory extends DefaultListenerFactory {

    //public constants
    protected static BCategory logger = BCategory.getBInstance(EventRedirectFactory.class.getName());

    //private vars
    protected String id = null;
    protected BaseEvent fevent = null;

    /**
     * Public constructor. Note that when actually forwarding the
     * event, a new instance will be generated via reflection.
     *
     * @param ifevent the event to be generated
     */
    public EventRedirectFactory (BaseEvent ifevent) {
        if (logger.isInfoEnabled()) logger.info("Creating new EventRedirectFactory -->"+ifevent.getClass().getName());
        fevent = ifevent;
    }

    /**
     * Get an instance of the underlying BaseEventListener
     *
     * @return get an instance of the BaseEventListener
     */
    public BaseEventListener getInstance() {
        return new EventHandler();
    }
    
    /**
     * Get the Listener ID associated with this class of listener. This will
     * generally either be the class name of the listener that the factory 
     * creates
     *
     * @return the listener ID that describes this factory
     */
    public String getListenerID() {
        return getID(fevent.getClass());
    }

    /**
     * EventHandler - 
     */
    class EventHandler extends DefaultBaseEventListener {

        protected String idStr = null;

        public void handleEvent(EventContext context) throws EventException {
            
            BaseEvent event = context.getEvent();
            DispatchQueue queue = context.getQueue();
            if (logger.isInfoEnabled()) logger.info("Got event:"+event);

            //in this case, we're not really doing anything special, so just
            //redirect to the RenderLogin view
            try {
                BaseEvent newEvent = (BaseEvent) fevent.getClass().newInstance();
                if (logger.isInfoEnabled()) logger.info("Redirecting to:"+newEvent);
                throw new ClientSideRedirectException(newEvent);
            } catch (InstantiationException e) {
                throw new EventException("Error redirecting Event", e);
            } catch (IllegalAccessException e) {
                throw new EventException("Error redirecting Event", e);
            }
        }
        
        /**
         * Get the ID that identifies this listener. This will typically be the 
         * class name.
         *
         * @return a string that uniquely identifies this listener
         */
        public String getListenerID() {
            if (idStr==null) {
                idStr = DefaultBaseEvent.getClassID(fevent.getClass());
            }
            return idStr;
        }
    }
}
