/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: And.java,v 1.6 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that two validators are BOTH valid, or a
 * ValidationException will be generated (effectively acting as
 * an "AND" or "&&")
 */
public class And extends AbstractFormValidator {

    protected FormValidator fv1 = null;
    protected FormValidator fv2 = null;
    
    /**
     * Public constructor.
     *
     * @param fv1 the first validator
     * @param fv2 the second validator
     */
    public And(FormValidator ifv1, FormValidator ifv2) {
        this(ifv1, ifv2, null);
    }

    /**
     * Public constructor.
     *
     * @param fv1 the first validator
     * @param fv2 the second validator
     * @param errmsg the message associated with this error
     */
    public And(FormValidator ifv1, FormValidator ifv2, String ierrmsg) {
        setErrorMessage(ierrmsg);
        fv1 = ifv1;
        fv2 = ifv2;
    }

    /**
     * Rather than calling all the sub validate methods, we make sure
     * both sub-validators are valid
     *
     * @param element the form element to be validated (null indicates
     *        we want to perform form level validation)
     * @param map the map to which the element belongs (sometimes necessary
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validate(FormElement element, FormMap map, boolean deferExceptions) throws ValidationException {
        if (localLogger.isDebugEnabled()) localLogger.debug("Making sure "+fv1+" and "+fv2+" are both valid");

        //validate fv1
        if (fv1==null) {
            throw this.generateException(element, deferExceptions,"Validator 1 was null");
        }
        try {
            fv1.validate(element, map, deferExceptions);
        } catch (ValidationException ve) {
            ValidationException veNew = this.generateException(element, deferExceptions, "Validator "+fv1+" was not valid");
            veNew.addSubException(ve);
            throw veNew;
        }

        //validate fv2
        if (fv2==null) {
            throw this.generateException(element, deferExceptions,"Validator 2 was null");
        }
        try {
            fv2.validate(element, map, deferExceptions);
        } catch (ValidationException ve) {
            ValidationException veNew = this.generateException(element, deferExceptions, "Validator "+fv2+" was not valid");
            veNew.addSubException(ve);
            throw veNew;
        }
    }
}
