/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Not.java,v 1.6 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that something is NOT valid. We do this
 * by validating another validator; if it does NOT generate a ValidationException
 * then we throw an exception because it should have. If it does throw a
 * ValidationException, then all is well. This validator effectively acts as
 * a "NOT" or "!" on another FormValidator.
 */
public class Not extends AbstractFormValidator {

    protected FormValidator fv = null;
    
    /**
     * Public constructor.
     *
     * @param fv the validator we wish to make sure is not valid
     */
    public Not(FormValidator ifv) {
        this(ifv, null);
    }

    /**
     * Public constructor.
     *
     * @param fv the validator we wish to make sure is not valid
     * @param errmsg the message associated with this error
     */
    public Not(FormValidator ifv, String ierrmsg) {
        setErrorMessage(ierrmsg);
        fv = ifv;
    }

    /**
     * Rather than calling all the sub validate methods, we instead validate
     * the validator we contain--because this validator is effectively doing
     * a "NOT", we expect to receive a ValidationException; if we dont, then
     * we will throw a ValidationException
     *
     * @param element the form element to be validated (null indicates
     *        we want to perform form level validation)
     * @param map the map to which the element belongs (sometimes necessary
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validate(FormElement element, FormMap map, boolean deferExceptions) throws ValidationException {
        if (localLogger.isDebugEnabled()) localLogger.debug("Making sure "+fv+" is not valid");

        //if fv==true, no error occurs, since a NOT of a null would be considered true...
        if (fv!=null) try {
            //validate the internal validator
            fv.validate(element, map, deferExceptions);
            
            //uh-oh; we didn't get an exception
            throw this.generateException(element, deferExceptions, "Validator "+fv+" was valid (and it shouldn't have been!)");
            
        } catch (ValidationException ve) {
            //ok to continue
        }
    }
}
