/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: EqualsValidator.java,v 1.6 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that the original value constitues a
 * valid type
 */
public class EqualsValidator extends DefaultFormValidator {

    protected Object obj = null;

    /**
     * Public no-args constructor.
     */
    public EqualsValidator() {
        this(null, null);
    }

    /**
     * Public constructor.
     *
     * @param obj the object with which we wish to compare equality
     */
    public EqualsValidator(Object iobj) {
        this(iobj, null);
    }

    /**
     * Public constructor.
     *
     * @param obj the object with which we wish to compare equality
     * @param errmsg the message associated with this error
     */
    public EqualsValidator(Object iobj, String ierrmsg) {
        super(ierrmsg);
        obj = iobj;
    }

    /**
     * Validate a FormElement to make see if the element equals() a
     * given object
     *
     * @param element the form element to be validated
     * @param element the form element that contains the val
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        if (localLogger.isInfoEnabled()) localLogger.info("validating to see if val {"+val+"} .equals() {"+obj+"}");
        // cannot validate a null value, use NotNullvalidator if you want to disallow nulls
        if (val==obj) return;
        if ((val==null && obj!=null) || (val!=null && !val.equals(obj))) {
            throw this.generateException(element, deferExceptions, "Value {"+val+"} is not equal to {"+obj+"}");
        }
    }
    
}
