/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ParamGateway.java,v 1.6 2001/10/03 21:25:20 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.helper.servlet;

import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.helper.state.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;

/**
 * <p>The purpose of this servlet is take a set of req parameters
 * and persist them in the users session. Then return the real target
 * url back to the calling application
 */
public class ParamGateway extends HttpServlet {

    //public vars...eventually, these should probably be final
    protected static BCategory logger = BCategory.getBInstance(ParamGateway.class.getName());

    //these are intentionally non-final, so that you can programatically
    //change them if need be
    public static String PARAM_TARGET = "pt_";
//    public static String PARAM_TARGET_ENC = "ptenc_";
    public static String PARAM_EXT = ".param_map";
    

    //-------------------- ParamGateway --------------------------
    /**
     * <p>Handle the default HttpRequest. 
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    public void handleDefault(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //persist the params
        if (logger.isDebugEnabled()) logger.debug("Persisting parameter information");
org.enhydra.barracuda.core.util.http.ServletUtil.showParams(req, logger);
        ParamPersister.persistReqParamState(req);
        
        //find the client context id
        String cid = req.getParameter("pm_cid");
        
        //figure out where we need to go from here by deriving it from 
        //the actual URL used to get us here
        String src = req.getRequestURI();
        if (logger.isDebugEnabled()) logger.debug("Request URI:"+src);
        int spos = src.indexOf(PARAM_TARGET);
        int epos = src.indexOf(PARAM_EXT);
//csc_100301.2_start
//        String target = src.substring(spos+PARAM_TARGET.length(), epos);
        String base = src.substring(spos+PARAM_TARGET.length(), epos);
        String param = src.substring(epos+PARAM_EXT.length());
        String target = base+param;
//csc_100301.2_end
        if (logger.isDebugEnabled()) logger.debug("Param Target:"+target);
        resp.setContentType("text/html");
        PrintWriter out = resp.getWriter();
        out.println("<html>");        
        out.println("  <head></head>");        
        out.println("  <body onload=\"p=document.layers?parentLayer:window.parent;p.jsrsLoaded('"+cid+"');\">");        
        out.println("    <form name=\"jsrs_Form\"><textarea name=\"jsrs_Payload\">"+target+"</textarea></form>");
        out.println("</html>");        
    }




    //-------------------- HTTPServlet ---------------------------
    /**
     * <p>By default the GET request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the POST request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the OPTIONS request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the DELETE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the PUT request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        handleDefault(req, resp);
    }
    
    /**
     * <p>By default the TRACE request is mapped to the handleDefault method
     *
     * @param req the servlet request
     * @param req the servlet response
     * @throws ServletException
     * @throws IOException
     */
    protected void doTrace(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {  
        handleDefault(req, resp);
    }
}
