/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: ParamPersister.java,v 1.2 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.helper.state;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.http.*;

/**
 * <p>Utility methods used for saving servlet request param State
 * and then reconstituting them later.
 */
public class ParamPersister {

    static final String PARAM_LIST = "ParamPersister.PARAM_LIST";

    /**
     * This method takes Request parameters and saves them in a
     * users session. End users should never need
     *
     * @param req the ServletRequest object
     */
    public static void persistReqParamState(HttpServletRequest req) {
        //eliminate the obvious
        if (req==null) return;

        //first see if there are any param values even associated
        //with this request
        HttpSession session = null;
        List paramList = null;

        //iterate through the list
        Enumeration enum = req.getParameterNames();
        while (enum.hasMoreElements()) {
            //get the session obj and create a list to store the values in
            if (session==null) {
                session = SessionServices.getSession(req);
                paramList = new ArrayList();
                session.setAttribute(PARAM_LIST, paramList);
            }

            //now get all the values associated with a given parameter
            //and save them in the state
            String key = (String) enum.nextElement();
            String[] values = req.getParameterValues(key);
            for (int i=0, max=values.length; i<max; i++) {
                paramList.add(new Param(key, values[i]));
            }        
        }
    }

    /**
     * This method reconstitutes Request parameters from a users session
     *
     * @param req the wrapper ServletRequest object
     */
    public static void reconstituteReqParamState(HttpServletRequestWrapper req) {
        //eliminate the obvious
        if (req==null) return;

        //see if the user even has a session. If not, there won't be any state
        HttpSession session = SessionServices.getSession(req, false);
        if (session==null) return;
        
        //now see if the session has any param state
        List paramList = (List) session.getAttribute(PARAM_LIST);
        if (paramList!=null) {
            //iterate through the list, adding them back into the req object
            Iterator it = paramList.iterator();
            while (it.hasNext()) {
                Param p = (Param) it.next();
                req.addParameter(p.getKey(), p.getValue());                
            }
        
            //finally make sure we clear out the param state
            session.removeAttribute(PARAM_LIST);
        }
    }
}
