/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: HttpSessionStateMap.java,v 1.3 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * The implementation provides a StateMap bridge to a HttpSession
 * object. By this, we mean that this class allows you to treat an
 * HttpSession as a StateMap. Unlike the HttpSession, this class 
 * will handle null keys, values.
 */
public class HttpSessionStateMap implements StateMap {

    //private vars
    protected HttpSession session = null;
    private String NULL = "~Null~";                    
    
    /**
     * Public constructor.
     *
     * @param session the underlying servlet session structure
     */    
    public HttpSessionStateMap(HttpSession isession) {
        session = isession;
    }
    
    /**
     * set a property in this StateMap
     *
     * @param key the key object
     * @param val the value object
     */    
    public void putState(Object key, Object val) {
        if (key==null) key = NULL;
        if (val==null) val = NULL;
        session.setAttribute(key.toString(),val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        if (key==null) key = NULL;
        Object val = session.getAttribute(key.toString());
        return (val.equals(NULL) ? null : val);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        if (key==null) key = NULL;
        Object val = session.getAttribute(key.toString());
        session.removeAttribute(key.toString());
        return (val.equals(NULL) ? null : val);
    }
    
    /**
     * get a List of the keys for this StateMap (implementation
     * is an ArrayList)
     *
     * @return a List the keys for this StateMap
     */    
    public List getStateKeys() {
        Enumeration e = session.getAttributeNames();
        List list = new ArrayList();
        while (e.hasMoreElements())    {
            list.add(e.nextElement());
        }        
        return list;
    }
    
    /**
     * get a copy of the underlying Map (in this case, we
     * map all the attributes of the HttpSession structure
     * into a Map and return that)
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        Map map = new HashMap();
        List keys = getStateKeys();
        Iterator it = keys.iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object val = session.getAttribute(key.toString());
            map.put(key,val);            
        }
        return map;        
    }

    /**
     * get a reference to the underlying HttpSession
     *
     * @return a reference to the underlying HttpSession
     */    
    public HttpSession getSession() {
        return session;
    }
}
