/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: ServletUtil.java,v 1.9 2001/10/03 21:25:20 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.http;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.logging.*;

/**
 * This just contains some Servlet utility routines
 */
public class ServletUtil {

    //-------------------- Utility stuff -------------------------
    /**
     * print all request info
     */
    public static void printAllRequestInfo (HttpServletRequest req, BCategory logger) {
        showGeneral(req, logger);
        showHeader(req, logger);
        showAttrs(req, logger);
        showParams(req, logger);
        showCookies(req, logger);
    }
    
    /**
     * print the general request info
     */
    public static void showGeneral (HttpServletRequest req, BCategory logger) {
        logger.debug(0, "General Request Info:");
        logger.debug(1, "Method:"+req.getMethod());
        logger.debug(1, "Protocol:"+req.getProtocol());
        logger.debug(1, "Scheme:"+req.getScheme());
        logger.debug(1, "AuthType:"+req.getAuthType());
        logger.debug(1, "IsSecure:"+req.isSecure());
        logger.debug(1, "ContextPath:"+req.getContextPath());
        logger.debug(1, "PathInfo:"+req.getPathInfo());
        logger.debug(1, "PathTranslated:"+req.getPathTranslated());
        logger.debug(1, "QueryString:"+req.getQueryString());
        logger.debug(1, "RequestURI:"+req.getRequestURI());
        logger.debug(1, "ServletPath:"+req.getServletPath());
    }
    
    /**
     * print the header request info
     */
    public static void showHeader (HttpServletRequest req, BCategory logger) {
        logger.debug(0, "Headers:");
        Enumeration enum = req.getHeaderNames();
        while (enum.hasMoreElements()) {
            String key = (String) enum.nextElement();
            String val = (String) req.getHeader(key);
            logger.debug(1, "name:"+key+" val:"+val);
        }
    }
    
    /**
     * print the attr request info
     */
    public static void showAttrs (HttpServletRequest req, BCategory logger) {
        logger.debug(0, "Attributes:");
        Enumeration enum2 = req.getAttributeNames();
        while (enum2.hasMoreElements()) {
            String key = (String) enum2.nextElement();
            String val = (String) req.getAttribute(key);
            logger.debug(1, "name:"+key+" attr:"+val);
        }
    }
    
    /**
     * print the param request info
     */
    public static void showParams (HttpServletRequest req, BCategory logger) {
        logger.debug(0, "Parameters:");
        Enumeration enum3 = req.getParameterNames();
        while (enum3.hasMoreElements()) {
            String key = (String) enum3.nextElement();
            String vals[] = req.getParameterValues(key);
            for (int i=0, max=vals.length; i<max; i++) {
                logger.debug(1, "key:"+key+" value:"+vals[i]);
            }
        }
    }
    
    /**
     * print the cookie request info
     */
    public static void showCookies (HttpServletRequest req, BCategory logger) {
        logger.debug(0, "Cookies:");
        Cookie[] cookies = req.getCookies();
        for (int i=0,max=cookies.length; i<max; i++) {
            Cookie c = cookies[i];
            logger.debug(1, "name:"+c.getName()+"  value:"+c.getValue()+"  ver:"+c.getVersion()+"  dom:"+c.getDomain()+"  comment:"+c.getComment()+"  max age:"+c.getMaxAge());
        }
    }

}    
