package org.enhydra.barracuda.core.util.logging;

import java.util.*;
import java.io.*;
import java.net.*;

import junit.framework.*;
import org.apache.log4j.*;

/**
 * Custom BCategory. This class simply provides a few convenience
 * methods on top of the regular Category functionality. Specifically.
 * it allows you to specify an indent.
 */
public class BCategory extends Category {

    public static String DEFAULT_LOG4J_FILE = "/log4j.properties";
    protected static BCategoryFactory bFactory = new BCategoryFactory();
    private static String spaces = "..............................................................................";
    private static boolean initialized = false;

    /**
     * The purpose of this static initializer is to configure
     * the log4j logging mechanism using the default option if it has
     * not already been configured somewhere else
     */
    static {
        // default configure the logger if not done so already
        if (!isInitialized()) {
            if (!isConfigured()) configure();
        initialized = true;
    }
    }

    protected BCategory(String name) {
        super(name);
    }
    
    public static boolean isInitialized() {
        return initialized;
    }
    
    public static void configure() {
        configure(DEFAULT_LOG4J_FILE);
    }
    
    private static boolean isConfigured() {
        Enumeration enum = Category.getRoot().getAllAppenders();

        if (!(enum instanceof org.apache.log4j.helpers.NullEnumeration)) {
            return true;
        } else {
            Enumeration cats =  Category.getCurrentCategories();
            while (cats.hasMoreElements()) {
                Category c = (Category) cats.nextElement();
                if (!(c.getAllAppenders() instanceof org.apache.log4j.helpers.NullEnumeration))
                    return true;
            }
        }
        return false;
    }

    public static void configure(String configFile) {
        URL log4jPropUrl = new BCategoryFactory().getClass().getResource(configFile);
        if (log4jPropUrl!=null) {
            System.out.println("Configuring Log4j from URL:"+log4jPropUrl);        
            PropertyConfigurator.configure(log4jPropUrl);
            initialized = true;
        } else {
            InputStream log4jIStream = BCategory.class.getClassLoader().getResourceAsStream(configFile);
            if (null != log4jIStream) {
                Properties log4jProperties = new Properties();
                try {
                    log4jProperties.load(log4jIStream);
                    System.out.println("Configuring Log4j from input stream:"+configFile);        
                    PropertyConfigurator.configure(log4jProperties);
                    initialized = true;
                } catch (IOException ex) {
                    System.out.println("Could not read log4j properties input stream");
                }
            } else {
                System.out.println("Configuring Log4j from File:"+configFile);        
                PropertyConfigurator.configure(configFile);
                initialized = true;
            }
        }
    }

    public static Category getInstance(String name) {
        return Category.getInstance(name, bFactory);
    }
    
    public static BCategory getBInstance(String name) {
        return (BCategory) BCategory.getInstance(name);
    }
    
    public void debug(int indent, Object message) {
        this.debug(inset(indent)+message);
    }
    
    public void debug(int indent, Object message, Throwable t) {
        this.debug(inset(indent)+message, t);
    }

    public void info(int indent, Object message) {
        this.info(inset(indent)+message);
    }
    
    public void info(int indent, Object message, Throwable t) {
        this.info(inset(indent)+message, t);
    }

    public void warn(int indent, Object message) {
        this.warn(inset(indent)+message);
    }
    
    public void warn(int indent, Object message, Throwable t) {
        this.warn(inset(indent)+message, t);
    }

    public void error(int indent, Object message) {
        this.error(inset(indent)+message);
    }
    
    public void error(int indent, Object message, Throwable t) {
        this.error(inset(indent)+message, t);
    }

    public void fatal(int indent, Object message) {
        this.fatal(inset(indent)+message);
    }
    
    public void fatal(int indent, Object message, Throwable t) {
        this.fatal(inset(indent)+message, t);
    }

    public String inset(int depth) {
        if (depth<0) depth = 0;
        if (depth>25) depth = 25;
        return spaces.substring(0,depth*3);
    }


}
