/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: GlobalUtilities.java,v 1.8 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.examples;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.event.*;


/**
 * Global Utilities. This class is primarily responsible for 
 * handling an HttpResponseEvent and generating a "Resource
 * not found error"
 */
public class GlobalUtilities extends DefaultEventGateway {

    //public constants

    //this defines the various event handlers
    private ListenerFactory httpResponseFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new HttpResponseHandler();}        public String getListenerID() {return getID(HttpResponseHandler.class);}};                

    //private vars
    private HttpResponseEvent lnkHttpRequestEvent;        //added by TJ to show association

    /**
     * Public constructor
     */
    public GlobalUtilities() {
        //specify who's interested in what
        specifyLocalEventInterests(httpResponseFactory, HttpResponseEvent.class);
    }

    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * HttpResponseHandler - this is the global handler for HttpResponseEvents.
     * If we get one of these events it means that something very unusual happened
     * in the system (like somebody requested an invalid event or something). So,
     * display a simple page indicating that the resource was not available.
     */
    class HttpResponseHandler extends DefaultBaseEventListener {
        public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletResponse resp = context.getResponse();

            //render the page
            resp.setContentType("text/html");
            PrintWriter out = resp.getWriter();
            out.println("<html><head><title>Resource not Found</title></head>");
            out.println("<body><font face=\"Arial\">");

            //display the general error
            String targetEventName = (String) context.getState(ApplicationGateway.TARGET_EVENT_NAME);
            out.println("<p>Beep-beep-beep! We're sorry your call did not go through. Will you please hang up and try your call again...");
            out.println("<p>If you received this message, it indicates that either <strong>"+targetEventName+"</strong> is not a valid event OR the system is misconfigured. If you feel there is a problem with the system please <a href=\"mailto:christianc@lutris.com\">contact the application administrator</a>... ");

            //log the specific problem
            BaseEvent be = DefaultBaseEvent.getRootEvent(event);
            out.println("<p>The base event that triggered this action was:"+be+"<br>");
            out.println("Source:"+be.getSource()+"<br>");
            
            //footer
            out.println("</font></body></html>");
        }
    }
}

