/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LoginScreen.java,v 1.11 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.examples.ex1;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.examples.ex1.events.*;
import org.enhydra.barracuda.examples.ex1.xmlc.*;


/**
 * Event handlers (both Controller and View) for the 
 * Login screen
 */
public class LoginScreen extends DefaultEventGateway {

    //public constants
    protected static BCategory logger = BCategory.getBInstance(LoginScreen.class.getName());

    //this defines the various event handlers
    private ListenerFactory getLoginFactory =               new EventForwardingFactory(new RenderLoginScreen());
    private ListenerFactory attemptLoginFactory =           new DefaultListenerFactory() {public BaseEventListener getInstance() {return new AttemptLoginHandler();}        public String getListenerID() {return getID(AttemptLoginHandler.class);}};                
    private ListenerFactory renderLoginFactory =            new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderLoginScreenHandler();}    public String getListenerID() {return getID(RenderLoginScreenHandler.class);}};                
    private ListenerFactory renderBadUsernameFactory =      new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderBadUsernameHandler();}    public String getListenerID() {return getID(RenderBadUsernameHandler.class);}};                
    private ListenerFactory renderBadPasswordFactory =      new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderBadPasswordHandler();}    public String getListenerID() {return getID(RenderBadPasswordHandler.class);}};                
    private ListenerFactory renderErrorFactory =            new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderErrorHandler();}            public String getListenerID() {return getID(RenderErrorHandler.class);}};                

    //private vars
    private AttemptLogin lnkAttemptLogin;                    //added by TJ to show association
    private GetLoginScreen lnkGetLoginScreen;                //added by TJ to show association
    private RenderLoginScreen lnkRenderLoginScreen;        //added by TJ to show association
    private BadUsername lnkBadUsername;                    //added by TJ to show association
    private BadPassword lnkBadPassword;                    //added by TJ to show association
    private ErrorEvent lnkError;                                  //added by TJ to show association

    /**
     * Public constructor
     */
    public LoginScreen() {
        //specify who's interested in what
        specifyLocalEventInterests(getLoginFactory, GetLoginScreen.class);
        specifyLocalEventInterests(attemptLoginFactory, AttemptLogin.class);
        specifyLocalEventInterests(renderLoginFactory, RenderLoginScreen.class);
        specifyLocalEventInterests(renderBadUsernameFactory, BadUsername.class);
        specifyLocalEventInterests(renderBadPasswordFactory, BadPassword.class);
        specifyLocalEventInterests(renderErrorFactory, ErrorEvent.class);
    }

    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * AttemptLoginHandler - this is where we handle any AttemptLogin 
     * events. Validate and then redirect accordingly.
     */
    class AttemptLoginHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletRequest req = context.getRequest();
            logger.debug("Attempting login...");

            try {
                //validate the login information
                int valid = LoginServices.validateUserPwd(req, true);
                logger.debug("Validity:"+LoginServices.xrefValidity(valid));

                //if it's valid, redirect to the Main screen
                if (valid==LoginServices.VALID) {
                    logger.debug("...Redirecting to Main Screen");
                    throw new ClientSideRedirectException(new GetMainScreen());
                //invalid user                
                } else if (valid==LoginServices.INVALID_USER) {
                    logger.debug("...Invalid User!");
                    throw new InterruptDispatchException(new BadUsername());
                //invalid pwd                
                } else if (valid==LoginServices.INVALID_PASSWORD) {
                    logger.debug("...Invalid Password!");
                    throw new InterruptDispatchException(new BadPassword());
                //anything else
                } else {
                    logger.debug("...Unknown Error!");
                    throw new InterruptDispatchException(new UnknownLoginError());
                }
            } finally {
                event.setHandled(true);
            }
        }
    }



    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderLoginScreenHandler - this is where we handle any RenderLoginScreen 
     * event and actually generate the view. We repoulate the form if there
     * is user/pwd information in the session.
     */
    class RenderLoginScreenHandler extends BTemplateViewHandler {
        public TemplateModel getTemplateModel() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return LoginHTML.class;}
    }

    /**
     * RenderBadUsernameHandler - this is where we handle any RenderBadUsername
     * event and actually generate the error view. We could just redisplay the
     * Login screen if we wanted and add an error message there, but I wanted to
     * demonstrate how we can throw InterruptDispatchExceptions to redirect
     * flow here.
     */
    class RenderBadUsernameHandler extends BTemplateViewHandler {
        public TemplateModel getTemplateModel() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return BadUsernameHTML.class;}
    }

    /**
     * RenderBadPasswordHandler - this is where we handle any RenderBadUsername
     * event and actually generate the error view. We could just redisplay the
     * Login screen if we wanted and add an error message there, but I wanted to
     * demonstrate how we can throw InterruptDispatchExceptions to redirect
     * flow here.
     */
    class RenderBadPasswordHandler extends BTemplateViewHandler {
        public TemplateModel getTemplateModel() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return BadPasswordHTML.class;}
    }

    /**
     * RenderErrorHandler - this is where we handle any Error event and generate a 
     * default view. Typically, this would only happen when a specific error was 
     * not caught and handled. In that case, the event proprogates up the event
     * chain, and we catch it here.
     */
    class RenderErrorHandler extends BTemplateViewHandler {
        public TemplateModel getTemplateModel() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return ErrorHTML.class;}
    }

    //------------------------------------------------------------
    //                Components - TemplateModel
    //------------------------------------------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
    
        //register the model by name
        public String getName() {return "Ex1_Login";}
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("GetLoginLink")) {
                return new BAction(new GetLoginScreen(), null);
            } else if (key.equals("AttemptLoginLink")) {
                return new BAction(new AttemptLogin(), null);
            } else if (key.equals("User")) {
                HttpSession session = SessionServices.getSession(vc, false);
                String user = (String) session.getAttribute(LoginServices.USER);
                if (user==null) user = "";
                return new BInput(null, LoginServices.USER, user, null, null);
            } else if (key.equals("Password")) {
                HttpSession session = SessionServices.getSession(vc, false);
                String password = (String) session.getAttribute(LoginServices.PASSWORD);
                if (password==null) password = "";
                return new BInput(null, LoginServices.PASSWORD, password, null, null);
            } else {
                return super.getItem(key);
            }
        }
    }
}

