/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestScreen.java,v 1.8 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.examples.ex3;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.examples.ex3.events.*;
//import org.enhydra.barracuda.experimental.html.*;


/**
 * This mini-app is just used for stress testing, to see what 
 * kind of performance level degradation we experience as the 
 * event heirarchy depth increases.
 *
 * The way this works is simple. If you invoke a Level1Event,
 * the dispatcher will fire a RequestEvent, HttpRequestEvent,
 * and then the actual Level1Event. If you invoke a Level10Event
 * there will be 9 additional events generated because of the 
 * Polymorphic nature of the request hierarchy.
 */
public class TestScreen extends DefaultEventGateway {

    //public constants
    protected static BCategory logger = BCategory.getBInstance(TestScreen.class.getName());

    //this defines the various event handlers
    private ListenerFactory levelFactory =                 new DefaultListenerFactory() {public BaseEventListener getInstance() {return new LevelHandler();}                public String getListenerID() {return getID(LevelHandler.class);}};                
    private ListenerFactory renderFactory =             new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderTestHandler();}            public String getListenerID() {return getID(RenderTestHandler.class);}};                

    //private vars

    /**
     * Public constructor
     */
    public TestScreen() {
        //specify who's interested in what
        specifyLocalEventInterests(levelFactory, Level1Event.class);
        specifyLocalEventInterests(levelFactory, Level2Event.class);
        specifyLocalEventInterests(levelFactory, Level3Event.class);
        specifyLocalEventInterests(levelFactory, Level4Event.class);
        specifyLocalEventInterests(levelFactory, Level5Event.class);
        specifyLocalEventInterests(levelFactory, Level6Event.class);
        specifyLocalEventInterests(levelFactory, Level7Event.class);
        specifyLocalEventInterests(levelFactory, Level8Event.class);
        specifyLocalEventInterests(levelFactory, Level9Event.class);
        specifyLocalEventInterests(levelFactory, Level10Event.class);
        specifyLocalEventInterests(renderFactory, RenderTestScreenEvent.class);
    }


    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * LevelHandlers - handle the request to get the test 
     * screen.
     */
    class LevelHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            DispatchQueue queue = context.getQueue();

            //in this case, we're not really doing anything special, so just
            //redirect to the RenderLogin view
            logger.debug("Handling event:"+event);
            BaseEvent newEvent = new RenderTestScreenEvent();
            newEvent.setSource(event);
            queue.addEvent(newEvent);
        }
    }
    

    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderTestHandler - handle the request to render the test 
     * screen
     */
    class RenderTestHandler extends DefaultBaseEventListener {
        public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletResponse resp = context.getResponse();

            //build the response
            resp.setContentType("text/html");
            PrintWriter out = resp.getWriter();
            
            //header            
            String title = "Simple Test Screen";
            out.println ("<html>");
            out.println ("  <head>");
            out.println ("    <title>"+title+"</title>");
            out.println ("  </head>");
            out.println ("  <body>");
            out.println ("    <font face=\"Arial\">");

            //body
            out.println ("      <p>"+title);
            
            //footer
            out.println ("    </font>");
            out.println ("  </body>");
            out.println ("</html>");
        }
    }
}

