package org.enhydra.barracuda.examples.xmlc;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.data.*;

/**
 * Check to see who Barracuda thinks we are
 */
public class BrowserCheck extends ComponentGateway {

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(BrowserCheckHTML.class);

        //create a template component and bind it to the views
        TemplateModel tm = new LocalTemplateModel(req);
        TemplateView tv = new DefaultTemplateView(page.getDocumentElement());
        BTemplate templateComp = new BTemplate(tm, tv);
        root.addChild(templateComp);

        //return the page
        return page;
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(true);
    }

    //--------------- TemplateModel --------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
    
        HttpServletRequest req = null;
        
        public LocalTemplateModel(HttpServletRequest ireq) {
            req = ireq;
        }
        
        //register the model by name
        public String getName() {return "BrowserCheck";}
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            ViewCapabilities vcap = vc.getViewCapabilities();
            if (key.equals("FormatType")) {
                return vcap.getFormatType();
            } else if (key.equals("ClientType")) {
                return vcap.getClientType();
            } else if (key.equals("ScriptingType")) {
                return vcap.getScriptingType();
            } else if (key.equals("User-Agent")) {
                return enumToString(req.getHeaders("User-Agent"));
            } else if (key.equals("Accept")) {
                return enumToString(req.getHeaders("Accept"));
            } else {
                return super.getItem(key);
            }
        }
        
        String enumToString(Enumeration enum) {
            StringBuffer sb = new StringBuffer(100);
            String sep = "";
            while (enum.hasMoreElements()) {
                sb.append(sep+enum.nextElement());
                sep = ",<br>";
            }
            return sb.toString();
        }
    }
}
