package org.enhydra.barracuda.examples.xmlc;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.data.*;

/**
 * Test TemplateHelper's ability to a) remove empty tags and 
 * b) support nested directives
 */
public class CompEx6 extends ComponentGateway {

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(CompEx6HTML.class);

        //create a template component and bind it to the views
        TemplateModel tm = new LocalTemplateModel(req);
        TemplateView tv = new DefaultTemplateView(page.getDocumentElement());
        BTemplate templateComp = new BTemplate(tm, tv);
        root.addChild(templateComp);

        //return the page
        return page;
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(true);
    }

    //--------------- TemplateModel --------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
    
        HttpServletRequest req = null;
        
        public LocalTemplateModel(HttpServletRequest ireq) {
            req = ireq;
        }
        
        //register the model by name
        public String getName() {return "Test1";}
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            ViewCapabilities vcap = vc.getViewCapabilities();
            if (key.equals("NullItem")) {
                return null;
            } else if (key.equals("Link1")) {
                BLink blink = new BLink("some text","someurl", vc);
                return blink;
            } else if (key.equals("Image1")) {
                return "/Barracuda/images/key.gif";
            } else if (key.equals("Markup1")) {
                try {
                    CompEx6aHTML snippetPage = (CompEx6aHTML) DefaultDOMLoader.getGlobalInstance().getDOM(CompEx6aHTML.class);
                    Node targetNode = snippetPage.getElementMarkupBlockID();
                    return vc.getElementFactory().getDocument().importNode(targetNode, true);                    
                } catch (IOException e) {
                    return "Unable to find "+key+" snippet:"+e;
                }
            } else if (key.equals("Text1")) {
                return "Some text goes here...";
            } else {
                return super.getItem(key);
            }
        }
    }
}
