package org.enhydra.barracuda.examples.xmlc;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.examples.xmlc.data.*;

/**
 * 
 */
public class RedirectEx1 extends ComponentGateway {

    public static boolean DEFAULT_NOBACK = true;    //default value

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(RedirectEx1HTML.class);

        //create a template component and bind it to the views
        TemplateModel tm = new SimpleTemplateModel(req, resp);
        TemplateView tv = new DefaultTemplateView(page.getElementById("TemplateID"));
        BTemplate templateComp = new BTemplate(tm, tv);
        root.addChild(templateComp);

        //add the appropriate headers to the response
//        resp.setHeader("Pragma","no-cache");
//        resp.setHeader("Cache-Control","no-cache");
//        resp.setDateHeader("Expires", System.currentTimeMillis());

        //return the page
        return page;
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
//        return new DefaultDOMWriter(true, true);
        return new DefaultDOMWriter(true);
    }

    //--------------- UserReportModel ------------------------
    /**
     * SimpleTemplateModel
     */
    class SimpleTemplateModel extends AbstractTemplateModel {
        HttpServletRequest req = null;
        HttpServletResponse resp = null;
        Integer cntr = null;
        Boolean noBack = null;
        String targetURL = "RedirectEx1";

        public SimpleTemplateModel(HttpServletRequest ireq, HttpServletResponse iresp) {
            req = ireq;
            resp = iresp;

            //adjust the cntr
            HttpSession session = req.getSession(true);
            cntr = (Integer) session.getAttribute("cntr");
            if (cntr==null || req.getParameter("clear")!=null) cntr = new Integer(0);
            else cntr = new Integer(cntr.intValue()+1);
            session.setAttribute("cntr", cntr);

            //figure out whether or not we are allowing back button
            noBack = (Boolean) session.getAttribute("noback");
            if (noBack==null) noBack = new Boolean(DEFAULT_NOBACK);
            if (req.getParameter("noback")!=null) noBack = new Boolean(true);
            else if (req.getParameter("okback")!=null) noBack = new Boolean(false);
            session.setAttribute("noback", noBack);
        }

        //register the model by name
        public String getName() {
            return "Rewrite";
        }
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
        
            //rewriting val        
            if (key.equals("RewritingVal")) {
                return new Boolean(URLRewriter.REWRITE_URLS);
                
            //back button val    
            } else if (key.equals("BackButtonVal")) {
                return noBack.booleanValue() ? "disabling" : "enabling";
                
            //servlet encoded url
            } else if (key.equals("ServletEncode")) {
//                String url = resp.encodeURL(targetURL+"?cntr="+cntr);
                String url = resp.encodeURL(targetURL);
                String text = "Target URL = "+url;
                BLink blink = new BLink(text, url, vc);
                blink.setDisableBackButton(noBack.booleanValue());
                return blink;
                
            //barracuda encoded url
            } else if (key.equals("BarracudaEncode")) {
//                String url = URLRewriter.encodeURL(req, resp, targetURL+"?cntr="+cntr);
                String url = URLRewriter.encodeURL(req, resp, targetURL);
                String text = "Target URL = "+url;
                BLink blink = new BLink(text, url, vc);
                blink.setDisableBackButton(noBack.booleanValue());
                return blink;
                
            //barracuda encoded url
            } else if (key.equals("EncodeForm1")) {
                String url = URLRewriter.encodeURL(req, resp, targetURL);
                BAction baction = new BAction(url, null);
                baction.setParam("foo","bar?bar/bar=bar~bar^bar,bar;bar");
                baction.setDisableBackButton(noBack.booleanValue());
                return baction;

            //barracuda encoded url
            } else if (key.equals("EncodeForm2")) {
                String url = URLRewriter.encodeURL(req, resp, targetURL);
                BAction baction = new BAction(url, null);
                baction.setDisableBackButton(noBack.booleanValue());
                return baction;

            //barracuda encoded url
            } else if (key.equals("EncodeForm3")) {
                String url = URLRewriter.encodeURL(req, resp, targetURL);
                BAction baction = new BAction(url, null);
                baction.setDisableBackButton(noBack.booleanValue());
                return baction;
//                BInput binput = new BInput(BInput.BUTTON, "name", "push me!", null, vc);
//                binput.setDisableBackButton(noBack.booleanValue());
//                return binput;

            //barracuda encoded url
            } else if (key.equals("EncodeForm4")) {
                String url = URLRewriter.encodeURL(req, resp, targetURL);
                BAction baction = new BAction(url, null);
                baction.setDisableBackButton(noBack.booleanValue());
                return baction;

            //session cntr
            } else if (key.equals("Cntr")) {
                return cntr;
                
            //reset cntr link
            } else if (key.equals("ResetLink")) {
                String text = "Reset Cntr";
                String url = URLRewriter.encodeURL(req, resp, targetURL+"?clear=true");
                BLink blink = new BLink(text, url, vc);
                blink.setDisableBackButton(noBack.booleanValue());
                return blink;
                
            //toggle back button link
            } else if (key.equals("ToggleLink")) {
                String text = (noBack.booleanValue() ? "Enable back button" : "Disable back button");
                String url = URLRewriter.encodeURL(req, resp, targetURL+(noBack.booleanValue() ? "?okback=true" : "?noback=true"));
                BLink blink = new BLink(text, url, vc);
                blink.setDisableBackButton(noBack.booleanValue());
                return blink;
                
            //anything else...
            } else return super.getItem(key);
        }
    }



    //xmlc factory (ok as static because its threadsafe)
//     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest.
     */
/*     
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        //adjust the cntr
        HttpSession session = req.getSession(true);
        Integer i = (Integer) session.getAttribute("cntr");
        if (i==null || req.getParameter("clear")!=null) i = new Integer(0);
        else i = new Integer(i.intValue()+1);
        session.setAttribute("cntr", i);

        //figure out whether or not we are allowing back button
        Boolean noBack = (Boolean) session.getAttribute("noback");
        if (noBack==null || req.getParameter("okback")!=null) noBack = new Boolean(false);
        else if (req.getParameter("noback")!=null) noBack = new Boolean(true);
        session.setAttribute("noback", noBack);

        //write a response
        resp.setContentType("text/html");
        PrintWriter out = resp.getWriter();
        out.println("<html>");            
        out.println("  <head><title>URL Rewrite Test</title></head>");            
        out.println("  <body>");            
        out.println("    <h1>URL Rewrite / Back Button Test</h1>");
        
        out.println("    <p>This example tests URL Rewriting in conjunction with disabling of the back button. Barracuda URL Rewriting is currently set to <strong>"+URLRewriter.REWRITE_URLS+"</strong>. To modify this value, you will need to adjust the URLRewriter.REWRITE_URLS variable.We are currently <strong>"+(noBack.booleanValue() ? "disabling" : "enabling")+"</strong> the back button");            

        String targetURL = "RedirectEx1";
        String encodedURL1a = resp.encodeURL(targetURL);
        String encodedURL1b = URLRewriter.encodeURL(req, resp, targetURL);
        String encodedURL2 = URLRewriter.encodeURL(req, resp, targetURL+"?clear=true");
        String encodedURL3 = URLRewriter.encodeURL(req, resp, targetURL+(noBack.booleanValue() ? "?okback=true" : "?noback=true"));
        
        out.println("    <p>Servlet Encoded URL:<a href=\""+encodedURL1a+"\">Target URL = "+encodedURL1a+"</a> (May not always work in older versions of Tomcat or Enhydra 3.x...if this link does not match the link below, you're server probably does not support implement the Servlet URL encoding methods properly.)</p>");            
        out.println("    <p>Barracuda Encoded URL:<a href=\""+encodedURL1b+"\">Target URL = "+encodedURL1b+"</a> </p>");            
        out.println("    <p>(Cntr = "+i+")");            
        out.println("    <p><a href=\""+encodedURL2+"\">(Reset cntr)</a></p>");            
        out.println("    <p><a href=\""+encodedURL3+"\">("+(noBack.booleanValue() ? "Enable back button" : "Disable back button")+")</a></p>");            
        out.println("  </body>");            
        out.println("</html>");            

    }
*/

}
