package org.enhydra.barracuda.examples.xmlc;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.xml.xmlc.*;
import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.data.*;
import org.enhydra.barracuda.examples.xmlc.events.*;


/**
 * A sample screen
 */
public class TemplateTableScreen extends DefaultEventGateway {

    //public constants
    protected static BCategory logger = BCategory.getBInstance(TemplateTableScreen.class.getName());

    //private constants
     private static String TEST_SCREEN = TemplateTableScreen.class+".TestScreen";

    //xmlc factory
     private static XMLCFactory xmlcFactory = null;

    //set up our factories
    private ListenerFactory getTestFactory =         new EventForwardingFactory(new RenderCompTest3());
    private ListenerFactory renderTestFactory =     new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderTestHandler();}            public String getListenerID() {return getID(RenderTestHandler.class);}};
    
    /**
     * Public constructor
     */
    public TemplateTableScreen() {
        //specify who's interested in what
        specifyLocalEventInterests(getTestFactory, CompTest3.class);
        specifyLocalEventInterests(renderTestFactory, RenderCompTest3.class);

        //set up the XMLCFactory
        xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
    }




    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderTestHandler - handle the request to render the test
     * screen
     */
    class RenderTestHandler extends DefaultBaseEventListener {
        public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletRequest req = context.getRequest();
            HttpServletResponse resp = context.getResponse();

            //set the response type
//            resp.setContentType("text/html");
//            PrintWriter out = resp.getWriter();

            //figure out which page we're after
            String pageName = ""+req.getParameter("page");
            Class pageCl = CompEx3HTML.class;
            if (pageName.equals("CompEx3a")) pageCl = CompEx3aHTML.class;
            else if (pageName.equals("CompEx3b")) pageCl = CompEx3bHTML.class;
            XMLObject page = xmlcFactory.create(pageCl);

            //create the component and bind it to the proper page (note that we're
            //doing this on a per-request basis...if we wanted to optimize we could
            //store the component hierarchy on a per-session basis)
            BComponent wcRoot = new BComponent();
            BTemplate wcTemplate = new BTemplate();
            wcTemplate.addView(new DefaultTemplateView(page.getDocument().getElementById("UserReport")));
            wcTemplate.addModel(new UserReportModel("org.enhydra.barracuda.examples.xmlc.UserReport"));            
            wcTemplate.addModel(new UserDataModel());            
            wcRoot.addChild(wcTemplate);
            
            //finally, render the component hierarchy
            try {
//                wcRoot.render(new ViewCapabilities());
                wcRoot.render(new DefaultViewContext(new ViewCapabilities()));
            } catch (RenderException e) {
                logger.warn("Error rendering views:",e);            
            }
            
            //now actually render the DOM
            logger.debug("Rendering document");            
//            out.println(page.toDocument());
//            new DefaultDOMWriter().write(page, resp.getWriter());
            new DefaultDOMWriter().write(page, resp);
        }
    }


    //--------------- UserReportModel ------------------------
    /**
     * UserReport Model
     */
    class UserReportModel extends DefaultPropertiesModel {

        CompEx3FooterHTML footer = (CompEx3FooterHTML) xmlcFactory.create(CompEx3FooterHTML.class);
        Node footerNode = footer.getElementUserReportFooter();

        /**
         * Instantiate the model by pointing it to the given
         * properties file (fully qualified name)
         */
        public UserReportModel(String propFileName) {
            super(propFileName);
        }

        /**
         * This is where you would return an item based on a given key.
         */
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            Object result = null;
            if (key.equals("Footer")) result = vc.getElementFactory().getDocument().importNode(footerNode, true);
            else result = (String) super.getItem(key);
            if (result==null) result = " ";
            return result;
        }

    }
    
    //--------------- UserDataModel --------------------------
    /**
     * UserData Model
     */
    class UserDataModel extends AbstractTemplateModel implements IterativeModel {

        //data structures
        UsersList users = null;
        UserData ud = null;
        Iterator it = null;

        //template nodes
        boolean useSelectionImages = true;
        Node selNode = null;
        Node unselNode = null;

        /**
         * Return the name of this model
         */
        public String getName() {return "UserData";}

        /**
         * This is where you would return an item based on a given key
         */
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            Object result = null;
            if (key.equals("Selected")) {
                if (useSelectionImages) {
                    ElementFactory ef = vc.getElementFactory();
                    selNode = ef.getElement("SelectedImageCell");
                    unselNode = ef.getElement("UnselectedImageCell");
                    useSelectionImages = false; //so we don't continually look for them if they're not there
                }
                boolean selected = ((Boolean) ud.get(UserData.SELECTED)).booleanValue();
                if (selNode!=null && selected) result = selNode.cloneNode(true);
                else if (unselNode!=null && !selected) result = unselNode.cloneNode(true);
                else result = (selected ? "Yes" : "No");
            }
            else if (key.equals("FirstName")) result = (String) ud.get(UserData.FIRST_NAME);
            else if (key.equals("LastName")) result = (String) ud.get(UserData.LAST_NAME);
            else if (key.equals("Gender")) result = (((Boolean) ud.get(UserData.GENDER)).booleanValue() ? "Male" : "Female");
            else if (key.equals("Age")) result = ""+ud.get(UserData.AGE);
            else if (key.equals("Phone")) result = (String) ud.get(UserData.PHONE);
            else if (key.equals("Email")) {
//                result = (String) ud.get(UserData.EMAIL);
                String email = (String) ud.get(UserData.EMAIL);
                result = new BLink(email, "mailto:"+email, vc);
            }
            else if (key.equals("Notes")) result = (String) ud.get(UserData.NOTES);
            else result = (String) super.getItem(key);
            if (result==null) result = " ";
            return result;
        }

        /**
         * Process any special directives
         */
        public boolean processDirective(TemplateDirective td) {
            if (td.getCommand().equals("Veto_If_Null")) {
                String data = td.getKeyName();
                if (data!=null) {
                    if (null != ud) {
                        StringTokenizer st = new StringTokenizer(data,",");
                        while (st.hasMoreTokens()) {
                            String key = st.nextToken();
                            Object value = ud.get(key);
                            if (value==null || value.toString().length()<1) return false; //don't allow this directive
                        }
                    } else {
                        return false; //don't allow this directive
                    }
                }
            }
            return true;    //by default, allow all directives
        }
        
        /**
         * Prepare for iteration
         */
        public void preIterate() {
            if (users==null) users = UsersList.getSampleInstance();
            it = users.iterator();
        }
        
        /**
         * Return true if there are more rows in the data model
         */
        public boolean hasNext() {
            return (it!=null && it.hasNext());
        }
        
        /**
         * Load the next row in the model
         */
        public void loadNext() {
            ud = (UserData) it.next();
        }
        
        /**
         * Post iteration cleanup
         */
        public void postIterate() {
            it = null;
        }
    }
    

}

