/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *  @author     christos@dynamic.net.au
 *  @author     kevin@bluedog.com.au
 *  @author     ericb@interactiveportal.com
 *
 * $Id: Xmlc.java,v 1.8 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.taskdefs;

import org.apache.tools.ant.*;
import org.apache.tools.ant.types.*;
import org.apache.tools.ant.types.*;
import java.lang.reflect.*;
import java.io.File;
import java.util.*;
import java.net.URL;
import java.net.MalformedURLException;
import org.apache.log4j.helpers.Loader;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.Category;

/**
 * ANT task to invoke the XMLC compiler to generate Java code.
 *
 *  <p>Implements an Ant taskdef to perform XMLC compilations. Please refer
 *  to the <a href="http://jakarta.apache.org/ant/index.html">Ant website</a>
 *  for more information. XMLCTask extends MatchingTask which means that
 *  it has the ability to include or exclude files based on pattern matching.
 *  The supported tags are as follows:</p>
 *
 *  <table border="1" cellspacing="3" cellpadding="3" width="100%">
 *    <tr>
 *      <th>tag</th>
 *      <th>required</th>
 *      <th>default value</th>
 *      <th>description</th>
 *    </tr>
 *    <tr>
 *      <td>srcdir</td>
 *      <td>yes</td>
 *      <td>&nbsp;</td>
 *      <td>The source directory root</td>
 *    </tr>
 *    <tr>
 *      <td>destdir</td>
 *      <td>no</td>
 *      <td>&nbsp;</td>
 *      <td>The destination directory root. Only really useful if you want XMLC to
 *          do the compilation, so we can check dependencies.
 *      </td>
 *    </tr>
 *    <tr>
 *      <td>packagedir</td>
 *      <td>no</td>
 *      <td>blank package</td>
 *      <td>Package prefix for generated classes, expressed as a directory path.
 *          If not specified, the relative path from srcdir becomes the package.
 *      </td>
 *    </tr>
 *    <tr>
 *      <td>markup</td>
 *      <td>no</td>
 *      <td>HTML</td>
 *      <td>The kind of markup language being processed.  Choices currently include
 *      HTML, WML, XHTML, CHTML and VoiceXML.  The generated class is suffixed with
 *      this value</td>
 *    </tr>
 *    <tr>
 *      <td>verbose</td>
 *      <td>no</td>
 *      <td>no</td>
 *      <td>Displays the XMLC command and all of the arguments</td>
 *    </tr>
 *    <tr>
 *      <td>compile</td>
 *      <td>no</td>
 *      <td>no</td>
 *      <td>Have XMLC compile the resulting java files? Note: The default is to not
 *          invoke the Java compiler through XMLC, since this is really slow. Any
 *          java files created are better handled by the ANT <javac> rule.
 *      </td>
 *    </tr>
 *    <tr>
 *      <td>keep</td>
 *      <td>no</td>
 *      <td>yes</td>
 *      <td>Keep the Java files generated by XMLC? To not keep them, you must allow XMLC to
 *          compile them first. However, since this is slower, the default is to keep them
 *          and let ANT compile the files using a seperate task.
 *      </td>
 *    </tr>
 *    <tr>
 *      <td>performExec</td>
 *      <td>no</td>
 *      <td>no</td>
 *      <td>XMLC is invoked by performing a Runtime.exec()<br><i>(kept for historical reasons by the author)</i></td>
 *    </tr>
 *    <tr>
 *      <td>options</td>
 *      <td>no</td>
 *      <td>&nbsp;</td>
 *      <td>File name of XMLC options or a glob style pattern.</td>
 *    </tr>
 *    <tr>
 *      <td>arg</td>
 *      <td>no</td>
 *      <td>&nbsp;</td>
 *      <td>Nested element representing additional command line arguments to be passed to XMLC.  See the XMLC documentation
 *          for all the possible commands. Note: these are applied after the other XMLC arguments derived from the options listed above.
 *          <p>For example, to specify an URL mapping to XMLC:
 *          <code>
 *            <pre>
 *            &lt;xmlc source="." includes="*.html"&gt;
 *              &lt;arg value="-urlmapping" /&gt;
 *              &lt;arg value="Edit.html" /&gt;
 *              &lt;arg value="To.po" /&gt;
 *            &lt;/xmlc&gt;
 *            </pre>
 *          </code>
 *          <b>Caution:</b> Do not be tempted to surround the URL's with single-quotes, as is common
 *          when calling XMLC from a shell command line, for example
 *            <pre>
 *            &lt;xmlc source="." includes="*.html"&gt;
 *              &lt;arg line="-urlmapping 'Edit.html' 'To.po'" /&gt;
 *            &lt;/xmlc&gt;
 *            </pre>
 *          The single-quotes will be taken as part of the URL to replace, and this is probably not what
 *          you intend
 *      </td>
 *    </tr>
 *
 *  </table>
 * <p>
 *
 *  @author     simon.tuffs@lutris.com
 *  @author     christos@dynamic.net.au
 *  @author     kevin@bluedog.com.au
 */
public class Xmlc extends org.apache.tools.ant.taskdefs.MatchingTask {

    // static initialization for this class
  static {
        System.setProperty(Category.DEFAULT_INIT_OVERRIDE_KEY, "true");
        final String log4jPropFileName = "file:log4j_localize.properties";
        URL log4jPropUrl = null;
        try {
            log4jPropUrl = new URL(log4jPropFileName);
        } catch (MalformedURLException ex) {
            // so, resource is not a URL:
            // attempt to get the resource from the class path
            log4jPropUrl =
                Loader.getResource(log4jPropFileName , Object.class);
        }    
        if (null != log4jPropUrl) {
            PropertyConfigurator.configure(log4jPropUrl);
        } else {
            System.out.println("Could not find log4j property file: " + log4jPropFileName);
        }
    }

    protected static String logHdr = "     [xmlc] ";

  /**
   *  Set the source directory containing the HTML files.
   *
   *  @param srcDirName The name of the directory containing the HTML files
   */
  public void setSrcdir(String srcdirname)
  {
    srcDirName = srcdirname;
    srcDir = project.resolveFile(srcDirName);
  }
  /**
   *  Set the destination directory into which the generated classes are
   *  written.
   *
   *  @param destDirName The destination directory into which the generated classes are written.
   */
  public void setDestdir(String destDirName)
  {
    destDir = project.resolveFile(destDirName);
  }
  /**
   * Force XMLC to always re-process the ML code (or not)
   */
  public void setForce(String force)
  {
    forceBuild = new Boolean(force).booleanValue();
  }
  /**
 * Allow general XMLC options to be passed through unchanged.  Appended to the
 * options lists.
 * @param args The XMLC options to be passed through.
 * @deprecated Use &lt;arg&gt; tags for now, this will eventually be removed.
 */
  public void setArgs(String args)
  {
    xmlcArgs = args;
    project.log(logHdr+"DEPRECATED, The 'args' attribute.  Use <arg> tags instead.", project.MSG_WARN);
  }

  /**
   * Set the classpath for the XMLC compiler.
   * @param classpath The classpath for XMLC to use.
   * @deprecated Use nested <arg> elements or "args" option instead.
   */
  public void setClasspath(String classpath)
  {
    classPath = classpath;
    project.log(logHdr+"DEPRECATED, The 'classpath' attribute.  Use <arg> tags instead.", project.MSG_WARN);
  }
  /**
   *  Set the destination directory into which the generated source .java files are
   *  written, if -keep is used.
   *
   *  @param sourceout The destination directory into which the generated sources are written.
   *
   * Note: Currently -keep is used by devault since the .java files are best
   * compiled by the built-in Ant javac command as a separate rule.
   */
  public void setSourceout(String sourceout)
  {
    srcOutDirName = sourceout;
    srcOutDir = project.resolveFile(srcOutDirName);
  }
  /**
   * Run XMLC in verbose mode (-verbose flag)
   * @param _verbose Set "true" for -verbose, anything else (e.g. "false") for
   * quiet mode.
   */
  public void setVerbose(BooleanAttribute _verbose)
  {
    verbose = (_verbose.getValue().equals("yes") || _verbose.getValue().equals("true"));
    // Set the project log level to INFO if the "verbose" argument is used in the Ant invocation
    // of this taskdef.
    if (verbose) {
      logLevel = project.MSG_INFO;
    } else {
      logLevel = project.MSG_VERBOSE;
    }
  }
  /**
   * Set the Package prefix for generated classes.  Expressed as a directory path.
   * @param packagedir Expressed as a directory path, either '/' or '\'
   * are accepted.<p>
   * Note: packagedir is used to set the package name of the generated class, <b>and</b>
   * to specify the directory where the file will be generated (relative to sourceout).
   * As such, any directory prefix on a source file is stripped before the packagedir
   * is applied.  This means that all files from the source tree will end up in the
   * same directory, which may not be what you intend, if you are using Xmlc to
   * process files in a directory hierarchy.
   * @see packagename
   */
  public void setPackagedir(String packagedir)
  {
    packageDir = packagedir;
  }

  /**
   * Set the package prefix for generated classes, while preserving any source
   * file directory paths.  This is useful if you want to take source (markup
   * language) files from one tree, but generate the Java code into another
   * tree, preserving relative paths.
   * <p>
   * For example, suppose that you have .html files under a directory called
   * "foo/resources", and you want to generate Java code under "foo/source",
   * but have the package name be "a.b". The following Xmlc taskdef will
   * achieve this:
   * <p>
   * <xmlc srcdir="foo/resources" sourceout="foo/source" packagename="a.b" />
   * <p>
   * A markup file located at "foo/resources/c/d/Foo.ml" will be generated to
   * "foo/source/a/b/c/d/FooML.java" and will start with a "package a.b.c.d"
   * statement.
   */
  public void setPackagename(String packagename)
  {
    packageName = packagename;
  }

  /**
   * The kind of markup language being processed.  Choices currently include
   * HTML, WML, XHTML, CHTML and VoiceXML.  The generated class is suffixed with
   * this value.
   * @param markup The type of markup language (controls only the file-name suffix,
   * the actual markup is controlled through the XMLC -dom option which is accessible
   * through the "args" option or &lt;arg&gt; tag.
   */
  public void setMarkup(String markup)
  {
    ML = markup;
  }
  /**
   *  Set whether or not to invoke XMLC by using Runtime.exec() to
   *  create a seperate JVM. This capability was originally written to get
   *  around a problem with name-space clashes between packages. Set this
   *  to the command to execute.
   *
   *  @param performExec Set to "true" or "false"
   */
  public void setPerformexec(String _performExec)
  {
    performExec = _performExec;
  }

  protected File srcDir, destDir, srcOutDir;
  protected String classPath, packageDir, packageName, srcDirName, srcOutDirName;
  protected String options = null;
  protected String xmlcArgs = null;
  protected boolean verbose, forceBuild;
  protected int logLevel = project.MSG_VERBOSE;
  protected int upToDate = 0;
  protected String performExec = null;
  // The following string is used as a suffix to the class name when generating
  // code.  This will vary, for HTML it defaults to "HTML", but other choices such
  // as "XML", "WML" etc would be appropriate.  This is set with the markup option.
  protected String ML = "HTML";

  /**
   *  Required by ant framework - implements execute() in Task.
   *  Called by the project to let the task do its work.
   *
   *  @exception BuildException All exceptions are thrown as BuildException.
   *
   * Note: most of the logic here was cloned from the Javac command. Unfortunately
   * this was the only way to 'reuse' it.
   */
  public void execute() throws org.apache.tools.ant.BuildException
  {
    project.log(logHdr+"Xmlc srcdir=" + srcDir, logLevel);
    project.log(logHdr+"     destdir=" + destDir, logLevel);
    project.log(logHdr+"     sourceout=" + srcOutDir, logLevel);
    project.log(logHdr+"     packagedir=" + packageDir, logLevel);
    project.log(logHdr+"     packagename=" + packageName, logLevel);
    // first off, make sure that we've got a srcdir

    if (srcDir == null) {
      throw new BuildException("srcdir attribute must be set!");
    }
    if (!srcDir.exists()) {
      throw new BuildException("srcdir does not exist!");
    }
    if (!compile && !keep) {
      throw new BuildException("If you do not keep the source, you must have XMLC compile them!");
    }

    XmlcUtils xmlcUtils = XmlcUtils.create();

    // scan source and dest dirs to build up both copy lists and
    // compile lists
    DirectoryScanner ds = this.getDirectoryScanner(srcDir);
    String[] files = ds.getIncludedFiles();

    project.log(logHdr+"XMLC processing " + files.length + " files: ", logLevel);
    for (int i=0; i<files.length; i++) {
      String file = files[i];
      try {
        String fileName = xmlcUtils.buildFullBaseFileName(packageName, packageDir, file);

        String className = xmlcUtils.buildClassName(fileName, ML);
        String javaName = className.replace('.', '/');

        // If srcOutDir is not specified, set it to srcDir.  This will put the
        // ML.java files in the same location as the .ml files, which is
        // desirable.
        if (srcOutDir == null) {
          srcOutDir = srcDir;
          srcOutDirName = srcDirName;
        }

        File mlFile = new File(srcDir, file);
//csc        File javaFile = new File(srcOutDir, fileName + ML + ".java");
        File javaFile = new File(srcOutDir, javaName + ".java");  //csc
        File classFile = new File((destDir != null) ? destDir : srcOutDir, fileName + ".class");

        java.util.ArrayList args = new java.util.ArrayList();

        // Quick test: if the .java file is newer than the .ml file, don't need
        // to run XMLC.  This is a rats-nest of logic, but hard to simplify.
        File target = null;

        if (keep) {
          if (!javaFile.exists()) {
            target = javaFile;
            // If we are going to keep the generated file but don't have one, then better create it
            project.log(logHdr+"    " + javaFile + " does not exist, invoking XMLC", logLevel);
          } else if (compile) {
            target = classFile;
            // If we are going to keep the generated file and compile it, then we have two dependencies to verify.
            if (mlFile.lastModified() <= classFile.lastModified() || mlFile.lastModified() <= javaFile.lastModified()) {
              if (!forceBuild) {
                project.log(logHdr+"    " + classFile.getName() + " or " + javaFile.getName() + " is newer than " + mlFile.getName(), logLevel);
                upToDate++;
                continue;
              }
            } else {
              project.log(logHdr+"    " + classFile.getName() + " or " + javaFile.getName() + " is older than " + mlFile.getName() + " invoking XMLC", logLevel);
            }
          } else {
            // If we are going to keep the generated file but not compile it, then we only care if the java
            // file is out of date. We'll let someone else check if the class file is out of date.
            target = javaFile;
            if (mlFile.lastModified() <= javaFile.lastModified()) {
              if (!forceBuild) {
                project.log(logHdr+"    " + javaFile.getName() + " is newer than " + mlFile.getName(), logLevel);
                upToDate++;
                continue;
              }
            } else {
              project.log(logHdr+"    " + javaFile.getName() + " is older than " + mlFile.getName() + " invoking XMLC", logLevel);
              target = javaFile;
            }
          }
        } else {
          // Not "keep" so target must be classFile.
          target = classFile;
          if (mlFile.lastModified() <= classFile.lastModified()) {
            if (!forceBuild) {
              // If we are not going to keep the generated file then we must compile it
              project.log(logHdr+"    " + classFile.getName() + " is newer than " + mlFile.getName(), logLevel);
              upToDate++;
              continue;
            }
          } else {
            project.log(logHdr+"    " + classFile.getName() + " is older than " + mlFile.getName() + " invoking XMLC", logLevel);
          }
        }
        project.log(logHdr+"    " + mlFile.getName() + " --> " + target.getName(), logLevel);
        if (classPath != null) {
          args.add("-classpath");
          args.add(classPath);
        }
        // Don't do the Java compilation here, it's much faster to do it with
        // the Javac task which invokes the Javac compiler without forking a
        // new process.
        if (!compile) args.add("-nocompile");
        if (keep) args.add("-keep");
        if (verbose) args.add("-verbose");

        args.add("-class");
        args.add(className);

        args.add("-sourceout");
        args.add(srcOutDir.toString());

        if (xmlcArgs != null) {
          // We have to tokenize the arguments.  Assumpions: URL's will not contain
          // whitespace, and if they did, they should use %20 or &space entities.
          //
          java.util.StringTokenizer tok = new java.util.StringTokenizer(xmlcArgs, " ");
          while (tok.hasMoreElements()) {
            args.add(tok.nextToken());
          }
        }

        String[] additionalArgs = cmdl.getArguments();
        for (int idx=0; idx < additionalArgs.length; idx++) {
          args.add(additionalArgs[idx]);
        }

        // The 'options' arg must  precede the name of the file to be compiled
        // Should really have -compile and -nokeep options since -nocompile
        // and -keep are the default.
        File[] possibleOptionFilesDirs = { javaFile.getParentFile(), mlFile.getParentFile()};
        String[] possibleOptionFileNames = { fileName, options};
        String[] foundOptionsFiles =
          xmlcUtils.getOptionFiles(possibleOptionFilesDirs, possibleOptionFileNames);

        if (foundOptionsFiles.length > 0) {
          for (int optionsIdx = 0; optionsIdx < foundOptionsFiles.length; optionsIdx++) {
            args.add(foundOptionsFiles[optionsIdx]);
            project.log(logHdr+"    Using options file " + foundOptionsFiles[optionsIdx], logLevel);
          }
        } else {
          project.log(logHdr+"No options file could not be located for " + file +
                      " in paths " + javaFile.getParentFile() + ", " +
                      mlFile.getParentFile(), project.MSG_WARN);
        }
        // *** WARNING DO NOT SEPARATE THE FOLLOWING CODE FROM THE BLOCK ABOVE! ***
        args.add(new File(srcDir, file).getAbsolutePath());

        project.log(logHdr+"args==" + args, logLevel);

        if (performExec != null) {
          args.add(0, performExec);
          String [] _args = (String[])args.toArray(new String[args.size()]);
          Runtime runtime = Runtime.getRuntime();
          Process process = runtime.exec(_args);
          int status = process.waitFor();
          if (status != 0)
            throw new BuildException("exit status of exec = "+status + toString(_args));
        } else {
          String [] _args = (String[])args.toArray(new String[args.size()]);

          // Execute XMLC directly.  This requires that enhydra.jar, or an equivalent
          // path containing the needed DOM code is on the CLASSPATH of ANT.
          String xmlcClassName = "org.enhydra.xml.xmlc.commands.xmlc.XMLC";

          Method      m = null;
          Class       c = null;
          try {
            c = Class.forName(xmlcClassName);
            m = c.getMethod("main", new Class[] { String[].class});

          } catch (Exception e) {
            // Reflection errors.  Class or method does not exist!
            e.printStackTrace();
            throw new BuildException(e);
          }

          if (m != null) {
            m.invoke(null,new Object[] { _args});
          }
        }
      } catch (InvocationTargetException ite) {
        ite.printStackTrace();
        throw new BuildException(ite);
      } catch (Exception x) {
        x.printStackTrace();
        throw new BuildException(x);
      }
    }
    if (upToDate > 0) {
      project.log(logHdr+"XMLC " + upToDate + " files were up-to-date, consider using force=\"true\"", logLevel);
    }
    /*
    // [TBD] I could not make this work, something subtle in Ant I don't grok.
    // Finally, do the Java compilation in one shot.
    project.log(logHdr+"Invoking Java Compiler");
    org.apache.tools.ant.taskdefs.Javac javac = new org.apache.tools.ant.taskdefs.Javac();
    javac.setSrcdir(srcDir.toString());
    javac.setDestdir(destDir.toString());
    javac.execute();
    // [TBD} If -keep not specified on the command line, clean out the java file.
    */
  }
  /**
   *  Simple utility to concatenate an array of Strings into one single
   *  String.
   *
   *  @param      args  The array of Strings
   *  @return     The resulting String
   */

  static private String toString(String[] args)
  {
    StringBuffer  buffer = new StringBuffer();

    for (int i=0; i<args.length; i++) buffer.append(args[i]+" ");
    return buffer.toString();
  }

  protected boolean keep = true;
  protected boolean compile = false;
  protected Commandline cmdl = new Commandline();

  /**
   *  Set whether or not to keep the generated Java file.
   *
   *  @param keep Set to "true" or "false"
   */
  public void setKeep(BooleanAttribute _keep)
  {
    keep = (_keep.getValue().equals("yes") || _keep.getValue().equals("true"));
  }
  /**
   *  Set whether or not to have XMLC compile the generated Java file.
   *
   *  @param compile Set to "true" or "false"
   */
  public void setCompile(BooleanAttribute _compile)
  {
    compile = (_compile.getValue().equals("yes") || _compile.getValue().equals("true"));
  }
  /**
   *  Set to a specific options file to add parameters to the XMLC command. Or,
   *  use a glob style pattern to base the options file name off the name of each
   *  source file to be processed (e.g. *.xmlc).
   *
   *  @param options Set to file name or glob pattern
   */
  public void setOptions(String _options)
  {
    options = _options;
  }
  /**
   *  Set nested arguments to the XMLC command to be executed.
   *
   */
  public Commandline.Argument createArg() {
    return cmdl.createArgument();
  }

  public static class BooleanAttribute extends EnumeratedAttribute {
    public String[] getValues() {
      return new String[] {"yes", "no", "true", "false"};
    }
  }

}
