package org.enhydra.barracuda.tutorials.comp;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.tutorials.xmlc.*;

/**
 * Here's an example of HelloWorld using the BTemplate
 * component
 */
public class HelloWorld1b extends ComponentGateway {

    //xmlc factory (ok as static because its threadsafe)
     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) {
        //load the DOM object
        if (xmlcFactory==null) xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
        XMLObject page = xmlcFactory.create(HelloWorld1bHTML.class);

        //find the node we're after
        Node node = page.getDocument().getElementById("HelloWorld");
        
        //create a view on the node
        TemplateView tv = new DefaultTemplateView(node);

        //create the model for the component
        TemplateModel tm = new HelloWorldModel();

        //create the component and bind it to the view and the model
        BTemplate templateComp = new BTemplate(tm, tv);

        //add the component to the root
        root.addChild(templateComp);
    
        //return the page
        return page;
    }

    //--------------- HelloWorldModel ------------------------
    /**
     * HelloWorldModel
     */
    class HelloWorldModel extends AbstractTemplateModel {
    
        //register the model by name
        public String getName() {
            return "HelloWorld";
        }
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("Title")) return "Hello World 1 example";
            else if (key.equals("Header")) return new BLink("Email Christian", "mailto:christian.cryder@lutris.com", vc);
            else if (key.equals("Descr")) return "This example illustrates how we can use models that return more than just Strings. Specifically, the model that backs the BTemplate component is returning a complex header and footer. For the header we are returning a BLink object that renders to the email address in the upper right hand corner. For the footer we are importing the HTML snippet that can be seen at the bottom of the page.";
            else if (key.equals("Hello")) return "Hello World! Hi Ma! Hi Pa!";
            else if (key.equals("Footer")) {
                XMLObject footer = xmlcFactory.create(HelloWorld1b_footerHTML.class);
                Node footerNode = footer.getElementById("Footer");
                return vc.getElementFactory().getDocument().importNode(footerNode, true);
            }
            else return super.getItem(key);
        }
    }
}
