package org.enhydra.barracuda.tutorials.comp;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.tutorials.xmlc.*;

/**
 * Here's an example of HelloWorld using the BList
 * component
 */
public class HelloWorld2a extends ComponentGateway {

    //xmlc factory (ok as static because its threadsafe)
     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) {
        //load the DOM object
        if (xmlcFactory==null) xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
        HelloWorld2aHTML page = (HelloWorld2aHTML) xmlcFactory.create(HelloWorld2aHTML.class);

        //start by setting basic data (here we just leverage existing 
        //XMLC technology)
        page.setTextTitle1("Hello World 2a example");
        page.setTextTitle2("Hello World 2a example");
        page.setTextDescr("This example illustrates how we can create populate a page using other Barracuda components--like BList--without putting anything in the HTML template except for standard XMLC style id values.");

        //ok, now we need to setup the list component and add 
        //it to the root
        String[] items = new String[] {"Cabbage", "Carrots", "Pickles", "Sugar", "Wheaties", "Flour", "Potatoes"};
        String[] qtys =  new String[] {"3 heads", "1 bag", "1 jar", "5 lbs", "3 bxs", "50 lbs", "25 lbs"};
        ListModel listModel = new GroceriesModel(items, qtys);
        View listView = new DefaultView(page.getElementShoppingList());
        BList wcList = new BList(listModel, listView);
        root.addChild(wcList);
        
        //set up the footer component (notice we use a list component)
        DefaultListModel flistModel = new DefaultListModel();
        View flistView = new DefaultView(page.getElementFooter());
        BList fwcList = new BList(flistModel, flistView);
        root.addChild(fwcList);

        //now to add data to the footer list we're actually going to 
        //add in all the child nodes
        Node fn = HelloWorld2Footer.getNode(page.getDocument());
        NodeList nl = fn.getChildNodes();
        for (int i=0,max=nl.getLength(); i<max; i++) {
            flistModel.add(nl.item(i));
        }
        
        //return the page
        return page;
    }

    //--------------- GroceriesModel -------------------------
    /**
     * Groceries Model
     */
    class GroceriesModel extends AbstractListModel {

        String[] items = null;
        String[] qtys = null;

        //constructor
        public GroceriesModel(String[] iitems, String[] iqtys) {
            items = iitems;
            qtys = iqtys;
        }

        //provide items by pos
        public Object getItemAt(int index) {
            return qtys[index]+" "+items[index];
        }

        //get the size of the list
        public int getSize() {
            return items.length;
        }
    }

}
