/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestExample.java,v 1.7 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda;

import java.util.*;

import junit.framework.*;

import org.enhydra.barracuda.core.util.logging.*;
import tests.util.*;


/**
 * Just a little sample test to illustrate how it all works, Runtime parameters
 * are defined in TestUtil.java. We are using basic Log4J logging, except that 
 * we are using a custom BCategory which extends the basic Category class. 
 * 
 * You can run individual test classes directly, or you can run the entire test 
 * suite using Ant. You control what gets logged by modifying the values in
 * /src/log4j.properties
 *
 * If you wish to create custom test classes, you should use this class as a 
 * model. 
 */
public class TestExample extends TestCase {
    //common vars (customize for every test class)
    private static String testClass = TestExample.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    private static boolean demonstrateFailure = false;        //change this to true to see how a failure is handled
    private static boolean demonstrateException = false;    //change this to true to see how an exception is handled
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestExample(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- TestCase ------------------------------
    /**
     * Set up the tests
     */
    protected void setUp() throws Exception {
        if (logger.isDebugEnabled()) logger.debug("Setting up test case");
        super.setUp();
        
        //this is where we can do any setup prior to 
        //actually running the tests if we so desire
        //(--n/a--)
    }

    /**
     * Tear down after the tests
     */
    protected void tearDown() throws Exception {
        if (logger.isDebugEnabled()) logger.debug("Tearing down test case");
        super.tearDown();
        
        //this is where we can do any cleanup after 
        //actually running the tests if we so desire
        //(--n/a--)
    }
    
    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Simple test to demonstrate how we can test various scenarios
     * and then log any errors which occur.
     */
    public void testSample() {
        if (logger.isInfoEnabled()) logger.info("running testSample()");

        //sample assertions (if an assertion fails then the error 
        //message will be thrown)
        try {
            //assert true!=false
            if (logger.isInfoEnabled()) logger.info(1, "testing true!=false");
            assert("true not equal false", true!=false);

            //assert 1==0
            if (demonstrateFailure) {
                if (logger.isInfoEnabled()) logger.info(1, "testing intentional failure");
                assert("intentionally failed assertion", 1==0);
            }
            
            //sample exception
            if (demonstrateException) {
                if (logger.isInfoEnabled()) logger.info(1, "generating sample exception");
                throw new Exception("Sample Exception");
            }

            //finished successfully!
            if (logger.isInfoEnabled()) logger.info(1, "tests complete!");
        } catch (Exception e) {
            //here's where we should catch unexpected exceptions and 
            //manually cause the test to fail
            logger.error(1, "Unexpected Test Exception", e);
            fail("Unexpected Test Exception:"+e);
        }
    }
}
