/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestBTemplateRender.java,v 1.3 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import tests.util.*;
import tests.util.servlet.*;


/**
 * This test verifies that BText works correctly.
 */
public class TestBTextRender extends TestCase {
    //common vars (customize for every test class)
    private static String testClass = TestBTextRender.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestBTextRender(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Simple test to verify that the component renders as expected
     */
    public void testBTextRender() throws IOException, RenderException {
        //render the doc
        if (logger.isInfoEnabled()) logger.info(1, "testing BText render");
        CompEx7 cg = new CompEx7();
        BComponent root = new BComponent();
        root.initCycle();

        //build a sample view capabilities object
        MockHttpServletRequest req = new MockHttpServletRequest();
        req.setHeader("User-Agent","Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)");
        ViewCapabilities vcap = new ViewCapabilities(req, null);
//System.out.println(vcap.toString());        

        //create the component amd render
        ViewContext vc = new DefaultViewContext(vcap);
        Document doc = cg.handleDefault(root, vc, req, null);
        root.render(vc);
        root.destroyCycle();
        DefaultDOMWriter ddw = new DefaultDOMWriter(false);
        StringWriter sw = new StringWriter(2000);
        ddw.write(doc, sw);
        String html = sw.toString();
//        ddw.write(doc, System.out);

        //assert that the text got rendered as we expected
        int fpos = html.indexOf("Text1: &lt;strong&gt;bold&lt;/strong&gt;");
        assert("BText render err1", fpos>0);
        fpos = html.indexOf("&lt;em&gt;zing1&lt;/em&gt;");
        assert("BText render err2", fpos>0);
        fpos = html.indexOf("Text2: <strong>bold</strong>");
        assert("BText render err3", fpos>0);
        fpos = html.indexOf("<em>zing2</em>");
        assert("BText render err4", fpos>0);
    }
}
