/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestCompCreate.java,v 1.4 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import tests.util.*;


/**
 * This test verifies that components get created correctly
 */
public class TestCompCreate extends TestCase {
    //common vars (customize for every test class)
    private static String testClass = TestCompCreate.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestCompCreate(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Simple test to make sure BComponents are instantiated correctly
     */
    public void testBComponentCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BComponent comp = new BComponent();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BText are instantiated correctly
     */
    public void testBTextCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BText comp = new BText();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BAction are instantiated correctly
     */
    public void testBActionCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BAction comp = new BAction();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BLink are instantiated correctly
     */
    public void testBLinkCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BLink comp = new BLink();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BInput are instantiated correctly
     */
    public void testBInputCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BInput comp = new BInput();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BToggleButton are instantiated correctly
     */
    public void testBToggleButtonCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BToggleButton comp = new BToggleButton();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BTemplate are instantiated correctly
     */
    public void testBTemplateCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BTemplate comp = new BTemplate();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BTable are instantiated correctly
     */
    public void testBTableCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BTable comp = new BTable();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BList are instantiated correctly
     */
    public void testBListCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BList comp = new BList();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    /**
     * Simple test to make sure BSelect are instantiated correctly
     */
    public void testBSelectCreate() {
        //make sure we can instantiate (ie. that its never declared abstract)
        BSelect comp = new BSelect();
    
        //ensure that the basic characteristics are as we expect
        performGeneralChecks(comp);
    }
    
    protected void performGeneralChecks(BComponent comp) {
        //make sure that a newly created component never has a name set. This is 
        //important, because if we ever start defaulting the name then it will
        //automatically set the name attribute in views (see the HTMLComponentRenderer
        //for details). The name attribute should only get set when the name is 
        //specifically set.
        assertNull("Default component name not null", comp.getName());
    }
}
