/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestDateRangeValidator.java,v 1.3 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;
import java.text.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.util.DateUtils;
import tests.util.*;

/**
 * This test verifies that DateRangeValidator works correctly.
 */
public class TestDateRangeValidator extends ValidatorTestCase {

	//common vars (customize for every test class)
	private static String testClass = TestDateRangeValidator.class.getName();
	private static BCategory logger = BCategory.getBInstance("test."+testClass);

	//variables
	
	//-------------------- Basics --------------------------------

	/**
	 * Public Constructor
	 */
	public TestDateRangeValidator(String name) {
		super(name);
	}
	
	/**
	 * Every test class should have a main method so it can be run
	 * directly (when debugging tests you often will not want to run
	 * the whole suite)
	 *
	 * @param args defined in test.util.TestUtil
	 */
	public static void main(String args[]) {
		//check for standard runtime parameters
		TestUtil.parseParams(args);

		//launch the test
		if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
		else junit.swingui.TestRunner.main(new String[] {testClass});
	}

	public static Test suite() {

		TestSuite allLocalesSuite = new TestSuite();

		Locale[] availableLocales = Locale.getAvailableLocales();
		for (int i = 0; i < availableLocales.length; i++) {
			TestSuite suite = new TestSuite();
			suite.addTestSuite( TestDateRangeValidator.class );
			allLocalesSuite.addTest( new LocaleTestSuiteSetup( suite, availableLocales[i]) );
			logger.warn("Adding:" + availableLocales[i].getCountry() );
		}

		return allLocalesSuite;
	}
	
	//-------------------- Actual Tests --------------------------
	//Note: all the methods herein should follow the testXXXX naming convention
	//Also keep in mind that local vars set in one test method are NOT retained
	//when the next method is invoked because JUnit makes a separate instance of
	//the test class for each testXXXX method!!!
	
	/**
	 * Test String types
	 */
	public void testString() {
		DateFormat aDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		aDateFormat.setLenient(false);
		DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

		Date now = new Date();
		DateRangeValidator afterNowValidator = new DateRangeValidator(now,null);

		assertAllInvalid(afterNowValidator, el,
									 new String[]{
											"   ",
											"foo",
											TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"),
											TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"),
											TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"),
											new String( aDateFormat.format( now ) ),
											new String( aDateFormat.format( DateUtils.getLastDateYesterday().getTime() ) ),

														 } );
		assertAllValid(afterNowValidator, el,
									 new String[]{
											null,
											"",
											TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1"),
											TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"),
											TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31"),
											new String( aDateFormat.format( DateUtils.getFirstDateTomorrow().getTime() ) ),
														 } );

		DateRangeValidator beforeNowValidator = new DateRangeValidator(null,now);

		assertAllInvalid(beforeNowValidator, el,
										 new String[]{
												"   ",
												"foo",
												TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1"),
												TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"),
												TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31"),
												new String( aDateFormat.format( DateUtils.getFirstDateTomorrow().getTime() ) ),
														 } );
		assertAllValid(beforeNowValidator, el,
									 new String[]{
											null,
											"",
											TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"),
											TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"),
											TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"),
											new String( aDateFormat.format( now ) ),
											new String( aDateFormat.format( DateUtils.getLastDateYesterday().getTime() ) ),
														 } );

		DateRangeValidator todayValidator = new DateRangeValidator( DateUtils.getFirstDateToday().getTime(),
																																DateUtils.getLastDateToday().getTime() );

		assertAllInvalid(todayValidator, el,
									 new String[]{
											"   ",
											"foo",
											TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"),
											TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"),
											TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"),
											TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"),
											new String( aDateFormat.format( DateUtils.getLastDateYesterday().getTime() ) ),
											new String( aDateFormat.format( DateUtils.getFirstDateTomorrow().getTime() ) ),
														 } );

		assertAllValid(todayValidator, el,
									 new String[]{
											null,
											"",
											new String( aDateFormat.format( new Date() ) ),
											new String( aDateFormat.format( DateUtils.getFirstDateToday().getTime() ) ),
											new String( aDateFormat.format( DateUtils.getLastDateToday().getTime() ) ),
														 } );
	}

	public void testLong() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);


		assertAllInvalid(v, el, new Long[]{ new Long(0),
																				new Long(1),
																				new Long(-1),
																				new Long(Long.MIN_VALUE),
																				new Long(Long.MAX_VALUE),
																			} );

		assertAllValid(v, el, new Long[]{ null, } );
	}
	
	public void testBoolean() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);

		assertAllInvalid(v, el, new Boolean[] { Boolean.FALSE,
																						Boolean.FALSE,
																					} );

		assertAllValid(v, el, new Boolean[]{ null, } );
	}
	
	public void testShort() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);
		assertAllInvalid(v, el, new Short[] { new Short((short)0),
																					new Short((short)1),
																					new Short((short)-1),
																					new Short(Short.MIN_VALUE),
																					new Short(Short.MAX_VALUE),
																				} );

		assertAllValid(v, el, new Short[]{ null, } );
	}
	
	public void testInteger() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);
		assertAllInvalid(v, el, new Integer[] { new Integer(0),
																						new Integer(1),
																						new Integer(-1),
																						new Integer(Integer.MIN_VALUE),
																						new Integer(Integer.MAX_VALUE),
																					} );

		assertAllValid(v, el, new Integer[]{ null, } );
	}
	
	public void testDate()  {
		DateFormat aDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		aDateFormat.setLenient(false);
		DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);

		Date now = new Date();
		DateRangeValidator afterNowValidator = new DateRangeValidator(now,null);

		try {
			assertAllInvalid(afterNowValidator, el,
										 new Date[]{
												new Date(now.getTime()-1),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")),
															 } );
			assertAllValid(afterNowValidator, el,
										 new Date[]{
												null,
												new Date(),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31")),
															 } );
		} catch ( java.text.ParseException ex )  {
			fail( "testDate failed because " + ex.getMessage() );
		}

		DateRangeValidator beforeNowValidator = new DateRangeValidator(null,now);

		try {
			assertAllInvalid(beforeNowValidator, el,
										 new Date[]{
												new Date(now.getTime()+1),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2003", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2999", "12", "31")),
															 } );
			assertAllValid(beforeNowValidator, el,
										 new Date[]{
												null,
												new Date(now.getTime()-1),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")),
															 } );
		} catch ( java.text.ParseException ex )  {
			fail( "testDate failed because " + ex.getMessage() );
		}

		DateRangeValidator todayValidator = new DateRangeValidator( DateUtils.getFirstDateToday().getTime(),
																																DateUtils.getLastDateToday().getTime() );
		try {
			assertAllInvalid(todayValidator, el,
										 new Date[]{
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")),
												DateUtils.getLastDateYesterday().getTime(),
												DateUtils.getFirstDateTomorrow().getTime(),
															 } );
			assertAllValid(todayValidator, el,
										 new Date[]{
												null,
												new Date(),
												DateUtils.getFirstDateToday().getTime(),
												DateUtils.getLastDateToday().getTime(),
															 } );
		} catch ( java.text.ParseException ex )  {
			fail( "testDate failed because " + ex.getMessage() );
		}
	}
	
	public void testDouble() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);
		assertAllInvalid(v, el, new Double[] {  new Double(0),
																						new Double(1),
																						new Double(-1),
																						new Double(Double.MIN_VALUE),
																						new Double(Double.MAX_VALUE),
																				 } );

		assertAllValid(v, el, new Double[]{ null, } );
	}
	
	public void testFloat() {
		DateRangeValidator v = new DateRangeValidator(new Date(),null);
		DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);
		assertAllInvalid(v, el, new Float[] { new Float(0),
																					new Float(1),
																					new Float(-1),
																					new Float(Float.MIN_VALUE),
																					new Float(Float.MAX_VALUE),
																				} );

		assertAllValid(v, el, new Float[]{ null, } );
	}

}
