/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestDateValidator.java,v 1.3 2001/09/25 19:05:30 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;
import java.text.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.logging.*;
import tests.util.*;

/**
 * This test verifies that DateValidator works correctly.
 */
public class TestDateValidator extends ValidatorTestCase {

	//common vars (customize for every test class)
	private static String testClass = TestDateValidator.class.getName();
	private static BCategory logger = BCategory.getBInstance("test."+testClass);

	//variables
	
	//-------------------- Basics --------------------------------

	/**
	 * Public Constructor
	 */
	public TestDateValidator(String name) {
		super(name);
	}
	
	/**
	 * Every test class should have a main method so it can be run
	 * directly (when debugging tests you often will not want to run
	 * the whole suite)
	 *
	 * @param args defined in test.util.TestUtil
	 */
	public static void main(String args[]) {
		//check for standard runtime parameters
		TestUtil.parseParams(args);

		//launch the test
		if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
		else junit.swingui.TestRunner.main(new String[] {testClass});
	}

	public static Test suite() {

		TestSuite allLocalesSuite = new TestSuite();

		Locale[] availableLocales = Locale.getAvailableLocales();
		for (int i = 0; i < availableLocales.length; i++) {
			TestSuite suite = new TestSuite();
			suite.addTestSuite( TestDateValidator.class );
			allLocalesSuite.addTest( new LocaleTestSuiteSetup( suite, availableLocales[i]) );
			logger.warn("Adding:" + availableLocales[i].getCountry() );
		}

		return allLocalesSuite;
	}
	
	//-------------------- Actual Tests --------------------------
	//Note: all the methods herein should follow the testXXXX naming convention
	//Also keep in mind that local vars set in one test method are NOT retained
	//when the next method is invoked because JUnit makes a separate instance of
	//the test class for each testXXXX method!!!
	
	/**
	 * Test String types
	 */
	public void testString() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

		assertAllInvalid(v, el,
										 new String[] {
																		"   ",
																		"foo",
																		TestUtil.dateStringInDefaultLocaleShortForm("2000", "1", "32"),
																		TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "1"),
																		TestUtil.dateStringInDefaultLocaleShortForm("2001", "2", "29"),

																		TestUtil.dateStringInDefaultLocaleShortForm("2000", "01", "32"),
																		TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "01"),
																		TestUtil.dateStringInDefaultLocaleShortForm("2001", "02", "29"),

																		TestUtil.dateStringInDefaultLocaleShortForm("2002", "01", "32"),
																		TestUtil.dateStringInDefaultLocaleShortForm("2010", "13", "01"),
																		TestUtil.dateStringInDefaultLocaleShortForm("2013", "02", "29"),
																	} );


		assertAllValid(v, el,
									 new String[] { "",
										              null,
																	TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2005", "1", "1"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2001", "12", "1"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"),
																	TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2010", "12", "31"),

																	TestUtil.dateStringInDefaultLocaleShortForm("2001", "01", "01"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2005", "01", "01"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2001", "12", "01"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2000", "02", "29"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2004", "02", "29"),
																} );
	}

	public void testLong() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);


		assertAllInvalid(v, el, new Long[]{ new Long(0),
																				new Long(1),
																				new Long(-1),
																				new Long(Long.MIN_VALUE),
																				new Long(Long.MAX_VALUE),
																			} );

		assertAllValid(v, el, new Long[]{ null, } );
	}
	
	public void testBoolean() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);

		assertAllInvalid(v, el, new Boolean[] { Boolean.FALSE,
																						Boolean.FALSE,
																					} );

		assertAllValid(v, el, new Boolean[]{ null, } );
	}
	
	public void testShort() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);
		assertAllInvalid(v, el, new Short[] { new Short((short)0),
																					new Short((short)1),
																					new Short((short)-1),
																					new Short(Short.MIN_VALUE),
																					new Short(Short.MAX_VALUE),
																				} );

		assertAllValid(v, el, new Short[]{ null, } );
	}
	
	public void testInteger() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);
		assertAllInvalid(v, el, new Integer[] { new Integer(0),
																						new Integer(1),
																						new Integer(-1),
																						new Integer(Integer.MIN_VALUE),
																						new Integer(Integer.MAX_VALUE),
																					} );

		assertAllValid(v, el, new Integer[]{ null, } );
	}
	
	public void testDate()  {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);

		DateFormat aDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		aDateFormat.setLenient(false);

		try {
			// NOTE - no invalid date test as it should not be possible to create an invalid date
			/*assertAllInvalid(v, el,
											 new Date[] {
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "1", "32")),
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "1")),
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "2", "29")),

													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "01", "32")),
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "01")),
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "02", "29")),
												} );*/

			assertAllValid(v, el,
										 new Date[]{
												null,
												new Date(),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2005", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "01", "01")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2005", "01", "1")),
															 } );
		} catch ( java.text.ParseException ex )  {
			fail( "testDate failed because " + ex.getMessage() );
		}
	}
	
	public void testDouble() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);
		assertAllInvalid(v, el, new Double[] {  new Double(0),
																						new Double(1),
																						new Double(-1),
																						new Double(Double.MIN_VALUE),
																						new Double(Double.MAX_VALUE),
																				 } );

		assertAllValid(v, el, new Double[]{ null, } );
	}
	
	public void testFloat() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);
		assertAllInvalid(v, el, new Float[] { new Float(0),
																					new Float(1),
																					new Float(-1),
																					new Float(Float.MIN_VALUE),
																					new Float(Float.MAX_VALUE),
																				} );

		assertAllValid(v, el, new Float[]{ null, } );
	}

}
