/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestValidTypeValidator.java,v 1.7 2001/09/28 23:11:08 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.logging.*;
import tests.util.*;


/**
 * This test verifies that ValidTypeValidator works correctly.
 */
public class TestValidTypeValidator extends ValidatorTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestValidTypeValidator.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestValidTypeValidator(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Test String types
     */
    public void testString() {
        //start by building a statemap
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("val_s1","foo");
        sm.putState("val_s2","null");
        sm.putState("val_s3",new Integer(123));

        checkIt(sm, FormType.STRING);
/*
        //now lets create a form map
        DefaultFormMap fm = new DefaultFormMap();
        Iterator it = sm.getKeys().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            fm.defineElement(new DefaultFormElement(key, FormType.STRING, null, new ValidTypeValidator()));
        }

        //mow map the StateMap into the FormMap        
        fm.map(sm);
        
        //now get all the form elements
        it = sm.getKeys().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            DefaultFormElement el = (DefaultFormElement) fm.getElement(key);
            DefaultFormValidator v = (DefaultFormValidator) el.getValidator();
            if (key.startsWith("val")) assertValid("Error validating element for key:"+key, v, el, null);
            else assertInvalid("Error invalidating element for key:"+key, v, el, null);
        }
*/        
    }
    
    /**
     * Test Boolean types - this test ensures that if we pass in a 
     * null FormElement or try to validate the length on a Boolean
     * it will always generate a ValidationException
     */
    public void testBoolean() {
        //start by building a statemap
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("val_b1","yes");
        sm.putState("val_b2","no");
        sm.putState("val_b3","true");
        sm.putState("val_b4","false");
        sm.putState("val_b5","on");
        sm.putState("val_b6","off");
        sm.putState("val_b7",null);
        sm.putState("inval_b8","boogle");
        sm.putState("inval_b9","blah");
        sm.putState("inval_b10","foo");

        checkIt(sm, FormType.BOOLEAN);
/*
        //now lets create a form map
        DefaultFormMap fm = new DefaultFormMap();
        Iterator it = sm.getKeys().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            fm.defineElement(new DefaultFormElement(key, FormType.BOOLEAN, null, new ValidTypeValidator()));
        }

        //mow map the StateMap into the FormMap        
        fm.map(sm);
        
        //now get all the form elements
        it = sm.getKeys().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            DefaultFormElement el = (DefaultFormElement) fm.getElement(key);
            DefaultFormValidator v = (DefaultFormValidator) el.getValidator();
            if (key.startsWith("val")) assertValid("Error validating element for key:"+key, v, el, null);
            else assertInvalid("Error invalidating element for key:"+key, v, el, null);
        }
*/
    }
    
    /**
     * Test Integer types
     */
    public void testInteger() {
        //start by building a statemap
        DefaultStateMap sm = new DefaultStateMap();
        sm.putState("val_i1","123");
        sm.putState("val_i2","123.0");
        sm.putState("inval_i3","3.141592654");
        sm.putState("val_i4",new Integer(123));
        sm.putState("val_i5",new Float(123));
        sm.putState("val_i6",new Double(123));
        sm.putState("val_i7",new Short((short) 123));
        sm.putState("inval_i8","13b");




//        sm.put("val_i3","true");
//        sm.put("val_i4","false");
//        sm.put("val_i5","on");
//        sm.put("val_i6","off");
//        sm.put("val_i7",null);
//        sm.put("inval_i8","boogle");
//        sm.put("inval_i9","blah");
//        sm.put("inval_i10","foo");

        checkIt(sm, FormType.INTEGER);
    }

    /**
     * Test Date types - this test ensures that if we try to validate 
     * the length on a Date it will always generate a ValidationException
     */
    public void testDate() {
        //todo
    }

    /**
     * Test Long types
     */
    public void testLong() {
        //todo
    }

    /**
     * Test Short types
     */
    public void testShort() {
        //todo
    }

    /**
     * Test Double types
     */
    public void testDouble() {
        //todo
    }

    /**
     * Test Float types
     */
    public void testFloat() {
        //todo
    }
    
    protected void checkIt(StateMap sm, FormType ft) {
        //now lets create a form map
        DefaultFormMap fm = new DefaultFormMap();
        Iterator it = sm.getStateKeys().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            fm.defineElement(new DefaultFormElement(key, ft, null, new ValidTypeValidator()));
        }

        //mow map the StateMap into the FormMap        
        fm.map(sm);
        
        //now get all the form elements
        it = sm.getStateKeys().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            DefaultFormElement el = (DefaultFormElement) fm.getElement(key);
            DefaultFormValidator v = (DefaultFormValidator) el.getValidator();
            if (key.startsWith("val")) assertValid("Error validating element for key:"+key, v, el, null);
            else assertInvalid("Error invalidating element for key:"+key, v, el, null);
        }
    }    
    
    
}
