/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: StateMapTestCases.java,v 1.3 2001/09/25 19:05:31 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.io.*;
import java.util.*;

import junit.framework.*;

import org.enhydra.barracuda.core.util.logging.*;
import tests.util.*;


/**
 * Test case for any StateMap. The idea is that you can test any
 * StateMap object by extending this test case and adding tests 
 * for the specific implementation details. All you really have 
 * to do is override teh factory StateMap methods.
 */
public abstract class StateMapTestCases extends TestCase {
    //common vars (customize for every test class)
    protected static String testClass = StateMapTestCases.class.getName();
    protected static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public StateMapTestCases(String name) {
        super(name);
    }
    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Verify the statemap aspects
     */
    public void testStateMap() {
        if (logger.isInfoEnabled()) logger.info(1, "testing statemap methods");
        StateMap map = getStateMap();
        String key1 = "foo1";
        String key2 = "foo2";
        String key3 = "foo3";
        String s1 = "blah1";
        Integer i1 = new Integer(99);
        String s2 = null;

        //simple tests
        map.putState(key1, s1);
        map.putState(key2, i1);
        map.putState(key3, s2);
        assert("Error 1a set/get state", map.getState(key1)==s1);
        assert("Error 1b set/get state", map.getState(key2)==i1);
        assert("Error 1c set/get state", map.getState(key3)==s2);
        List keys = map.getStateKeys();
        assert("Error 2a - Key list length is wrong", keys.size()==3);
        assert("Error 2b - Key missing", keys.contains(key1));        
        assert("Error 2c - Key missing", keys.contains(key2));        
        assert("Error 2d - Key missing", keys.contains(key3));        
        Map values = map.getStateValues();
        assert("Error 3a - Value list length is wrong", values.size()==3);
        assert("Error 3b - Value missing", values.get(key1)==s1);        
        assert("Error 3c - Value missing", values.get(key2)==i1);        
        assert("Error 3d - Value missing", values.get(key3)==s2);        
        map.removeState(key1);
        map.removeState(key2);
        map.removeState(key3);
        assert("Error 4a remove state", map.getState(key1)==null);
        assert("Error 4b remove state", map.getState(key2)==null);
        assert("Error 4c remove state", map.getState(key3)==null);
    }
    
    
    //-------------------- Abstract methods ----------------------
    public abstract StateMap getStateMap();

}
