/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestPArrayList.java,v 1.3 2001/09/25 19:05:31 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.util.*;

import junit.framework.*;

import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.logging.*;
import tests.util.*;


/**
 * Test case for any PList
 */
public class TestPArrayList extends PListTestCases {
    //common vars (customize for every test class)
    static {
        testClass = TestPArrayList.class.getName();
        logger = BCategory.getBInstance("test."+testClass);
    }
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestPArrayList(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    //NOTE: most of the individual tests are actually inherited 
    //from the parent class

    /**
     * Verify the clone (we have to test this here instead
     * the clone method is not part of the List interface...we need
     * a concrete implementation in order to be able to reference it)
     */
    public void testClone() {
        if (logger.isInfoEnabled()) logger.info(1, "testing clone()");

        //test an empty list
        PArrayList plist1 = (PArrayList) this.getPListInstance();
        PArrayList plist2 = (PArrayList) plist1.clone();
        assert("Cloned obj==source obj at 1", plist1!=plist2);
        assertEquals("Clone check 1a failed", plist1, plist2);
        assertEquals("Clone check 1b failed", plist2, plist1);

        //test a list with some items
        plist1 = (PArrayList) this.getPListInstance();
        plist1.add("foo1");
        plist1.add("foo2");
        plist1.add("foo3");
        plist1.add(new Integer(99));
        plist1.add(null);
        plist2 = (PArrayList) plist1.clone();
        assert("Cloned obj==source obj at 2", plist1!=plist2);
        assertEquals("Clone check 2a failed", plist1, plist2);
        assertEquals("Clone check 2b failed", plist2, plist1);
        
        //test a list with some PData items
        plist1 = (PArrayList) this.getPListInstance();
        plist1.add("foo1");
        PList plTmp = this.getPListInstance();
        plTmp.add("blah 1");
        plTmp.add("blah 2");
        plist1.add(plTmp);
        plist2 = (PArrayList) plist1.clone();
        assert("Cloned obj==source obj at 3", plist1!=plist2);
        assertEquals("Clone check 3a failed", plist1, plist2);
        assertEquals("Clone check 3b failed", plist2, plist1);
    }

    //-------------------- Abstract methods ----------------------
    public StateMap getStateMap() {
        return getPListInstance();
    }

    public PData getPDataInstance() {
        return getPListInstance();
    }

    public PList getPListInstance() {
        return new PArrayList();
    }

    public PMap getPMapInstance() {
        return new PHashMap();
    }
}
