/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestParam.java,v 1.3 2001/09/25 19:05:31 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import tests.util.*;


/**
 * This test verifies the Param object
 */
public class TestParam extends TestCase {
    //common vars (customize for every test class)
    private static String testClass = TestParam.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables

    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestParam(String name) {
        super(name);
    }

    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }


    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Simple test to make sure the Param object works
     */
    public void testParam() {
        if (logger.isInfoEnabled()) logger.info(1, "testing param");

        //vars
        Param p = null;
        Param p2 = null;
        String key1 = "foo1";
        String key1a = "foo1";
        String key2 = "foo2";
        String value1 = "blah1";
        String value1a = "blah1";
        String value2 = "blah2";

        //basics
        p = new Param();
        assert("param obj check 1a.1 failed - p.key!=null", p.key==null);
        assert("param obj check 1a.2 failed - p.getKey()!=null", p.getKey()==null);
        assert("param obj check 1b.1 failed - p.value!=null", p.value==null);
        assert("param obj check 1b.2 failed - p.getvalue()!=null", p.getValue()==null);
        p = new Param(key1, value1);
        assert("param obj check 2a.1 failed - p.key!=key", p.key==key1);
        assert("param obj check 2a.2 failed - p.getKey()!=key", p.getKey()==key1);
        assert("param obj check 2b.1 failed - p.value!=value", p.value==value1);
        assert("param obj check 2b.2 failed - p.getvalue()!=value", p.getValue()==value1);

        //equality (shouldn't happen until both key and value match)
        p2 = new Param(key2, value2);
        assert("param obj check 3a.1 failed - equality check failed", !(p.equals(p2)));
        assert("param obj check 3a.2 failed - equality check failed", !(p2.equals(p)));
        p2.setKey(key1);
        assert("param obj check 3a.3 failed - equality check failed", !(p.equals(p2)));
        assert("param obj check 3a.4 failed - equality check failed", !(p2.equals(p)));
        p2.setKey(key2);
        p2.setValue(value1);
        assert("param obj check 3a.5 failed - equality check failed", !(p.equals(p2)));
        assert("param obj check 3a.6 failed - equality check failed", !(p2.equals(p)));
        p2.setKey(key1);
        assert("param obj check 3a.7 failed - equality check failed", (p.equals(p2)));
        assert("param obj check 3a.8 failed - equality check failed", (p2.equals(p)));
        p2.setKey(key1a);
        p2.setValue(value1a);
        assert("param obj check 3a.9 failed - equality check failed", (p.equals(p2)));
        assert("param obj check 3a.10 failed - equality check failed", (p2.equals(p)));
    }
}
