/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestClientType.java,v 1.3 2001/09/25 19:05:31 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.view;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.logging.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import tests.util.*;
import tests.util.servlet.*;


/**
 * This test verifies that the hierachical organization of 
 * ClientType classes. We do this by testing for what it should be 
 * an instanceof and what it shouldn't. The idea is that the ClientType
 * extension hierarchy is very important and we don't want it getting 
 * changed accidentally. This test should preclude that.
 */
public class TestClientType extends TestCase {
    //common vars (customize for every test class)
    private static String testClass = TestClientType.class.getName();
    private static BCategory logger = BCategory.getBInstance("test."+testClass);

    //variables
    ClientType ct = null;
    Class targetCl = null;
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestClientType(String name) {
        super(name);
    }

    public TestClientType(String name, ClientType ict, Class itargetCl) {
        super(name);
        ct = ict;
        targetCl = itargetCl;
    }
    
    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    /**
     * Return a suite of tests...
     */
	public static Test suite() {
        //create the test suite
		TestSuite suite = new TestSuite();
        
        //add the tests
        //...HTML_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_BROWSER, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_BROWSER, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_BROWSER, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_BROWSER, ClientType.UnknownBrowser.class));
        //...HTML_3_2_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_3_2_BROWSER, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_3_2_BROWSER, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_3_2_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_3_2_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_3_2_BROWSER, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_3_2_BROWSER, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_3_2_BROWSER, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_3_2_BROWSER, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_3_2_BROWSER, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_3_2_BROWSER, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_3_2_BROWSER, ClientType.UnknownBrowser.class));
        //...IE_3x
        suite.addTest(new TestClientType("testInstance", ClientType.IE_3x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_3x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_3x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_3x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_3x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_3x, ClientType.IE4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_3x, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_3x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_3x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_3x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_3x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_3x, ClientType.UnknownBrowser.class));
        //...IE_4x
        suite.addTest(new TestClientType("testInstance", ClientType.IE_4x, ClientType.IE4x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_4x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_4x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_4x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_4x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_4x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_4x, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_4x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_4x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_4x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_4x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_4x, ClientType.UnknownBrowser.class));
        //...NN_3x
        suite.addTest(new TestClientType("testInstance", ClientType.NN_3x, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_3x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_3x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_3x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_3x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_3x, ClientType.NN4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_3x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_3x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_3x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_3x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_3x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_3x, ClientType.UnknownBrowser.class));
        //...NN_4x
        suite.addTest(new TestClientType("testInstance", ClientType.NN_4x, ClientType.NN4x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_4x, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_4x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_4x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_4x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_4x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_4x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_4x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_4x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_4x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_4x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_4x, ClientType.UnknownBrowser.class));
        //...OPERA_4x
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_4x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_4x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_4x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_4x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_4x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_4x, ClientType.Opera5x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_4x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_4x, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_4x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_4x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_4x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_4x, ClientType.UnknownBrowser.class));
        //...HTML_4_0_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_4_0_BROWSER, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_4_0_BROWSER, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_4_0_BROWSER, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_4_0_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.HTML_4_0_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.IE5x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.NN6x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.Opera5x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.HTML_4_0_BROWSER, ClientType.UnknownBrowser.class));
        //...IE_5x
        suite.addTest(new TestClientType("testInstance", ClientType.IE_5x, ClientType.IE5x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_5x, ClientType.IE4x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_5x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_5x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_5x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_5x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_5x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_5x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_5x, ClientType.IE6x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_5x, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_5x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_5x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_5x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_5x, ClientType.UnknownBrowser.class));
        //...IE_6x
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.IE6x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.IE5x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.IE4x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.IE_6x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_6x, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_6x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_6x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_6x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.IE_6x, ClientType.UnknownBrowser.class));
        //...NN_6x
        suite.addTest(new TestClientType("testInstance", ClientType.NN_6x, ClientType.NN6x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_6x, ClientType.NN4x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_6x, ClientType.NN3x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_6x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_6x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_6x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_6x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.NN_6x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_6x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_6x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_6x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_6x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.NN_6x, ClientType.UnknownBrowser.class));
        //...OPERA_5x
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_5x, ClientType.Opera5x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_5x, ClientType.Opera4x.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_5x, ClientType.Html40Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_5x, ClientType.Html32Browser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_5x, ClientType.HtmlStandardBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_5x, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.OPERA_5x, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_5x, ClientType.IE3x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_5x, ClientType.NN4x.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_5x, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_5x, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.OPERA_5x, ClientType.UnknownBrowser.class));
        //...CHTML_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.CHTML_BROWSER, ClientType.ChtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.CHTML_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.CHTML_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.CHTML_BROWSER, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.CHTML_BROWSER, ClientType.UnknownBrowser.class));
        //...XML_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.XML_BROWSER, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.XML_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.XML_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.XML_BROWSER, ClientType.UnknownBrowser.class));
        //...VXML_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.VXML_BROWSER, ClientType.VxmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.VXML_BROWSER, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.VXML_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.VXML_BROWSER, ClientType.WmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.VXML_BROWSER, ClientType.XhtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.VXML_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.VXML_BROWSER, ClientType.UnknownBrowser.class));
        //...WML_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.WML_BROWSER, ClientType.WmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.WML_BROWSER, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.WML_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.WML_BROWSER, ClientType.VxmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.WML_BROWSER, ClientType.XhtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.WML_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.WML_BROWSER, ClientType.UnknownBrowser.class));
        //...HXTML_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.XHTML_BROWSER, ClientType.XhtmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.XHTML_BROWSER, ClientType.XmlBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.XHTML_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.XHTML_BROWSER, ClientType.VxmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.XHTML_BROWSER, ClientType.WmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.XHTML_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.XHTML_BROWSER, ClientType.UnknownBrowser.class));
        //...UNKNOWN_BROWSER
        suite.addTest(new TestClientType("testInstance", ClientType.UNKNOWN_BROWSER, ClientType.UnknownBrowser.class));
        suite.addTest(new TestClientType("testInstance", ClientType.UNKNOWN_BROWSER, ClientType.GenericBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.UNKNOWN_BROWSER, ClientType.HtmlBrowser.class));
        suite.addTest(new TestClientType("testNotInstance", ClientType.UNKNOWN_BROWSER, ClientType.XmlBrowser.class));

        //finally return the suite
        return suite;
    }


    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Make sure that the class being tested IS an instance of the 
     * target class
     */
    public void testInstance() {
        if (logger.isInfoEnabled()) logger.info(1, "testing instanceof");
        assert(ct+" not an instanceof "+targetCl, targetCl.isAssignableFrom(ct.getClass()));
    }

    /**
     * Make sure that the class being tested is NOT an instance of the 
     * target class
     */
    public void testNotInstance() {
        if (logger.isInfoEnabled()) logger.info(1, "testing !instanceof");
        assert(ct+" is an instanceof "+targetCl+" (and it shouldn't be!)", !targetCl.isAssignableFrom(ct.getClass()));
    }
}
