/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestUtil.java,v 1.10 2001/09/25 19:05:31 cryd0221 Exp $
 */
package tests.util;

import org.apache.log4j.*;
import java.util.*;
import java.text.*;

/**
 * This class just defines basic constants (BATCH_MODE, DEBUG_LEVEL)
 * and parses runtime parameters to set those constants.
 */
public class TestUtil {

    //other runtime parameters
    public static boolean BATCH_MODE = false;
    public static int DEBUG_LEVEL = 0;

    /**
     * This utility method is used to parse an argument string
     * to automatically adjust any global paramters
     *
     * Known paramters:
     * ---------------
     * batch - (opt) run the test in batch mode (as opposed to interactive Swing mode,
     *        which is the default)
     * debug_level - (opt) an int value which determines whether debug msgs show
     */
    public static void parseParams (String args[]) {
        //check to see runtime parameters
        if (args!=null) {
            for (int i=0; i<args.length; i++) {
                String parm = args[i].toLowerCase();
                if (parm.startsWith("batch=")) {
                    BATCH_MODE = (parm.toLowerCase().indexOf("true")>0);
                } else if (parm.startsWith("debug_level=")) {
                    try  {DEBUG_LEVEL = Integer.parseInt(parm.substring(12));}
                    catch (Exception e) {}
                }
            }
        }

        //show what we've got
        System.out.println ("Setting BATCH_MODE="+TestUtil.BATCH_MODE);
        System.out.println ("Setting DEBUG_LEVEL="+TestUtil.DEBUG_LEVEL);
        System.out.println ("");
    }

    public static String dateStringInDefaultLocaleShortForm(String theYear, String theMonth, String theDay) {
        Calendar aCalendar = Calendar.getInstance();
        // zero everything
        aCalendar.clear();
        aCalendar.set(2000, 0, 1);

        Date aZeroDate = aCalendar.getTime();
        DateFormat aDefaultDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
        FieldPosition yearFieldPosn = new FieldPosition(DateFormat.YEAR_FIELD);
        FieldPosition monthFieldPosn = new FieldPosition(DateFormat.MONTH_FIELD);
        FieldPosition dayFieldPosn = new FieldPosition(DateFormat.DATE_FIELD);
        aDefaultDateFormat.format(aZeroDate, new StringBuffer(), yearFieldPosn );
        aDefaultDateFormat.format(aZeroDate, new StringBuffer(), monthFieldPosn );
        aDefaultDateFormat.format(aZeroDate, new StringBuffer(), dayFieldPosn );

        // sort the field positions into descending start index order
        ArrayList sortedPositions = new ArrayList();
        sortedPositions.add(yearFieldPosn);
        if ( monthFieldPosn.getBeginIndex() > yearFieldPosn.getBeginIndex() ) {
            sortedPositions.add(0, monthFieldPosn);
        } else {
            sortedPositions.add(monthFieldPosn);
        }
        if ( dayFieldPosn.getBeginIndex() > ((FieldPosition)sortedPositions.get(0)).getBeginIndex() ) {
            sortedPositions.add(0, dayFieldPosn);
        } else if (dayFieldPosn.getBeginIndex() > ((FieldPosition)sortedPositions.get(1)).getBeginIndex() ) {
            sortedPositions.add(1, dayFieldPosn);
        } else {
            sortedPositions.add(dayFieldPosn);
        }

        // create string buffer with formatted current date in it (to make it sure is correct length)
        StringBuffer aStrBuff = new StringBuffer( aDefaultDateFormat.format(aZeroDate) );
        for ( int i = 0; i < sortedPositions.size(); i++ ) {
            FieldPosition currFieldPosn = (FieldPosition)sortedPositions.get(i);
            String currField = theYear;
            if ( currFieldPosn == monthFieldPosn ) {
                currField = theMonth;
            } else if ( currFieldPosn == dayFieldPosn) {
                currField = theDay;
            }
            aStrBuff.replace(currFieldPosn.getBeginIndex(), currFieldPosn.getEndIndex(), currField);
        }

        return ( aStrBuff.toString() );
    }
}
