/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: Debug.java,v 1.4 2001/08/24 20:43:41 cryd0221 Exp $
 */
package tests.util;

import java.io.*;
import java.util.*;

//This class should no longer be used...


/**
 * Simple debugging utilities
 */
public class Debug {

    //debug state settings
    public static final int SUPPRESS_OUTPUT = 0;
    public static final int NORMAL = 1;
    public static final int SHOW_SOURCE = 2;
    public static final int ADD_TIMESTAMP = 3;
    public static final int ADD_SEQUENCE_CNTR = 4;

    //public vars
    public static int showDebug = SHOW_SOURCE;

    //private vars
    private static String pad = "                              ";
    private static String spaces = "..............................................................................";
    private static PrintStream ps = System.out;
    private static int cntr = 0;

    /**
     * Print a line to System.out, indenting based on depth.
     * It will show the class generating the debug message, 
     * and optionally add a timestamp (useful for debugging 
     * multi threaded situations)
     */
    public static void println(Object source, int depth, String s) {
        if (showDebug==SUPPRESS_OUTPUT) return;
        if (depth<0) depth = 0;
        if (depth>25) depth = 25;
        String inset = spaces.substring(0,depth*3);
        String sourceStr = "";
        if (showDebug>=SHOW_SOURCE) {
            sourceStr = source.getClass().getName()+"-->";
            int epos = sourceStr.lastIndexOf('.');
            if (epos>-1) sourceStr = sourceStr.substring(epos+1);
            if (sourceStr.length()>30) sourceStr = sourceStr.substring(0,27)+"-->";
            if (sourceStr.length()<30) sourceStr = sourceStr+pad.substring(0,30-sourceStr.length());
        }
        String timeStr = "";
        if (showDebug>=ADD_TIMESTAMP) {
            timeStr = " ["+System.currentTimeMillis()+"]";
        }
        String seqStr = "";
        if (showDebug>=ADD_SEQUENCE_CNTR) {
            seqStr = "["+(++cntr)+"] ";
        }
        ps.println (seqStr+sourceStr+inset+s+timeStr);            
        ps.flush();
    }

    /**
     * Print a stack trace to the debug console
     *
     * @param th the Throwable who's stack trace we wish to see 
     */
    public static void printStackTrace(Throwable th) {
        th.printStackTrace(getDebugPrintStream());
    }

    /**
     * Get debug PrintStream
     *
     * @return the debug PrintStream object (usually System.out)
     */
    public static PrintStream getDebugPrintStream() {
        return ps;
    }
}